@@ -31,21 +31,22 @@
 
 
 
-#ifndef __BGEOT_CONFIG_H
-#define __BGEOT_CONFIG_H
+#ifndef BGEOT_CONFIG_H__
+#define BGEOT_CONFIG_H__
 
 #include <getfem_arch_config.h>
-
 #include <dal_except.h>
+#include <gmm_kernel.h>
+#include <gmm_dense_lu.h>
 
 #ifdef GETFEM_HAVE_QDLIB
-#  define NO_INLINE
+// #  define NO_INLINE
 #  ifdef GETFEM_QDLIB_USE_QUAD
-#    include <qd.h>
+#    include <qd/qd.h>
 #  else
-#    include <dd.h>
+#    include <qd/dd.h>
 #  endif
-#  include <x86.h>
+#  include <qd/fpu.h>
 #endif
 
 namespace bgeot
@@ -58,16 +59,19 @@ namespace bgeot
 #ifndef GETFEM_HAVE_QDLIB
   typedef double long_scalar_type;
   typedef double opt_long_scalar_type;
+# define LONG_SCALAR_ATOF(st) (atof(st))
 # define LONG_SCALAR_EPS 1E-16
 # define LONG_SCAL(xx) long_scalar_type(xx)
 #else
 #  ifdef GETFEM_QDLIB_USE_QUAD
   typedef qd_real long_scalar_type;
   typedef qd_real opt_long_scalar_type;
+# define LONG_SCALAR_ATOF(st) (long_scalar_type(st))
 # define LONG_SCALAR_EPS 1E-64
 #  else
   typedef dd_real long_scalar_type;
   typedef dd_real opt_long_scalar_type;
+# define LONG_SCALAR_ATOF(st) (long_scalar_type(st))
 # define LONG_SCALAR_EPS 1E-32
 #  endif
 #  define LONG_SCAL(xx) long_scalar_type(#xx) /* string assignment to preserve the precision */
@@ -82,4 +86,4 @@ namespace bgeot
 }  /* end of namespace bgeot.                                             */
 
 
-#endif /* __BGEOT_CONFIG_H                                                */
+#endif /* BGEOT_CONFIG_H__                                                */
