@@ -32,8 +32,8 @@
 
 
 
-#ifndef __BGEOT_GEOMETRIC_TRANSFORMATION_H
-#define __BGEOT_GEOMETRIC_TRANSFORMATION_H
+#ifndef BGEOT_GEOMETRIC_TRANSFORMATION_H__
+#define BGEOT_GEOMETRIC_TRANSFORMATION_H__
 
 #include <bgeot_config.h>
 #include <bgeot_convex_ref.h>
@@ -63,10 +63,10 @@ namespace bgeot
    *     which should be a diffeomorphism between $\overline{T}$ and $T$. It
    *     is assumed that there exists a polynomial vector \\
    *     $ \underline{\cal N}(\overline{x})
-   *        = \left({\cal N}_i(\overline{x})\right)_i, \ \ i = 0 .. n_g-1, $ \\
+   *        = \left({\cal N}i_(\overline{x})\right)i_, \ \ i = 0 .. n_g-1, $ \\
    *     defined on $\overline{T}$ of size $n_g$, such that the transformation
    *     $\tau$ can be written \\
-   *     $ \tau(\overline{x}) = \sum_{i = 0}^{n_g-1} {\cal N}_i(\overline{x})
+   *     $ \tau(\overline{x}) = \sum_{i = 0}^{n_g-1} {\cal N}i_(\overline{x})
    *     g^i. $ \\
    *     Denoting by \\
    *     $ \underline{\underline{G}} = (g^0; g^1; ...;g^{n_g-1}), $ \\
@@ -77,9 +77,9 @@ namespace bgeot
    *   \subsubsection*{Gradient of the transformation}
    *     The gradient of the transformation is \\
    *     $ \nabla \tau(\overline{x}) = 
-   *     \left( \frac{\partial \tau_i}{\partial \overline{x}_j} \right)_{ij}
+   *     \left( \frac{\partial \tau_i}{\partial \overline{x}j_} \right)_{ij}
    *     = \left( \sum_{l = 0}^{n_g-1}g^l_i
-   *     \frac{\partial {\cal N}_l(\overline{x})}{\partial \overline{x}_j}
+   *     \frac{\partial {\cal N}l_(\overline{x})}{\partial \overline{x}j_}
    *     \right)_{ij} = \underline{\underline{G}}\ \nabla
    *     \underline{\cal N}(\overline{x}), $ \\
    *     Remark : $\underline{\underline{G}}$ is a $N \times n_g$ matrix,
@@ -117,7 +117,7 @@ namespace bgeot
     const std::vector<base_node> &geometric_nodes(void) const
     { return cvr->points(); }
     /// Gives the array of the normals to faces.
-    const std::vector<base_vector> &normals(void) const
+    const std::vector<base_small_vector> &normals(void) const
     { return cvr->normals(); }
     
     template<class CONT> base_node transform(const base_node &pt,
@@ -131,7 +131,8 @@ namespace bgeot
     base_node P(ptab[0].size()); P.fill(0.0);
     size_type k = nb_points();
     for (size_type l = 0; l < k; ++l)
-      P.addmul(poly_vector()[l].eval(pt.begin()),ptab[l]);
+      gmm::add(gmm::scaled(ptab[l],poly_vector()[l].eval(pt.begin())),P);
+      //P.addmul(poly_vector()[l].eval(pt.begin()),ptab[l]);
     return P;
   }
 
@@ -143,8 +144,12 @@ namespace bgeot
 
   pgeometric_trans simplex_geotrans(size_type n, short_type k);
   pgeometric_trans parallelepiped_geotrans(size_type n, short_type k);
+  pgeometric_trans parallelepiped_linear_geotrans(size_type n);
   pgeometric_trans prism_geotrans(size_type n, short_type k);
-  pgeometric_trans product_geotrans(pgeometric_trans pg1, pgeometric_trans pg2);
+  pgeometric_trans prism_linear_geotrans(size_type n);
+  pgeometric_trans product_geotrans(pgeometric_trans pg1,pgeometric_trans pg2);
+  pgeometric_trans linear_product_geotrans(pgeometric_trans pg1,
+					   pgeometric_trans pg2);
 
   pgeometric_trans geometric_trans_descriptor(std::string name);
   /* List :
@@ -163,12 +168,12 @@ namespace bgeot
 
      pt is the position of the evaluation point on the reference element
   */
-  base_vector compute_normal(const base_matrix &G, size_type ir,
-			     pgeometric_trans pgt, const base_node &pt);
+  base_small_vector compute_normal(const base_matrix &G, size_type ir,
+				   pgeometric_trans pgt, const base_node &pt);
 
    //@}
 
 }  /* end of namespace bgeot.                                             */
 
 
-#endif /* __BGEOT_GEOMETRIC_TRANSFORMATION_H                              */
+#endif /* BGEOT_GEOMETRIC_TRANSFORMATION_H__                              */
