@@ -31,8 +31,8 @@
 /* *********************************************************************** */
 
 
-#ifndef __BGEOT_POLY_H
-#define __BGEOT_POLY_H
+#ifndef BGEOT_POLY_H__
+#define BGEOT_POLY_H__
 
 #include <bgeot_config.h>
 #include <vector>
@@ -55,9 +55,21 @@ namespace bgeot
   /** Vector of integer (16 bits type) which represent the powers
    *  of a monomial
    */
-  class power_index : public std::vector<short_type> {
+  class power_index {
+    std::vector<short_type> v;
+    mutable short_type degree_;
+    mutable size_type global_index_;
+    void dirty() const { degree_ = short_type(-1); global_index_ = size_type(-1); }
   public :
-    
+    typedef std::vector<short_type>::iterator iterator;
+    typedef std::vector<short_type>::const_iterator const_iterator;
+    short_type operator[](size_type idx) const { return v[idx]; }
+    short_type& operator[](size_type idx) { dirty(); return v[idx]; }
+    iterator begin() { dirty(); return v.begin(); }
+    const_iterator begin() const { return v.begin(); }
+    iterator end() { dirty(); return v.end(); }
+    const_iterator end() const { return v.end(); }
+    size_type size() const { return v.size(); }
     /// Gives the next power index 
     const power_index &operator ++();
     /// Gives the next power index 
@@ -77,7 +89,7 @@ namespace bgeot
     /// Constructor
     power_index(short_type nn);
     /// Constructor
-    power_index(void) {}
+    power_index(void) { dirty(); }
   };
   
   /**
@@ -111,7 +123,7 @@ namespace bgeot
    *       If two monomials have the same degree, they are ordered with the
    *       degree of the mononomials without the n firsts variables which
    *       have the same degree. The index of the monomial
-   *       $ x _0^{i_0}x_1^{i_1} ... x_{n-1}^{i_{n-1}} $
+   *       $ x 0_^{i_0}x_1^{i_1} ... x_{n-1}^{i_{n-1}} $
    *       is then
    *       $ \alpha_{d-1}^{n} + \alpha_{d-i_0-1}^{n-1} 
    *          + \alpha_{d-i_0-i_1-1}^{n-2} + ... + \alpha_{i_{n-1}-1}^{1}, $
@@ -139,7 +151,7 @@ namespace bgeot
    *     \subsubsection*{Horner scheme to evaluate polynomials}
    *        To do ...
    */
-  template<class T> class polynomial : public std::vector<T> {
+  template<typename T> class polynomial : public std::vector<T> {
   protected :
     
     short_type n, d;
@@ -198,31 +210,35 @@ namespace bgeot
     /// Makes P = 1.
     void one(void) { change_degree(0); (*this)[0] = T(1); }
     void clear(void) { change_degree(0); (*this)[0] = T(0); }
-    template <class ITER> T horner(power_index &mi, short_type k,
+    template <typename ITER> T horner(power_index &mi, short_type k,
 				   short_type de, const ITER &it) const;
     /** Evaluate the polynomial. "it" is an iterator pointing to the list
      * of variables. A Horner scheme is used.
      */
-    template <class ITER> T eval(const ITER &it) const;
+    template <typename ITER> T eval(const ITER &it) const;
 
     /// Constructor.
     polynomial(void) : std::vector<T>(1)
       { n = 0; d = 0; (*this)[0] = 0.0; }
     /// Constructor.
-    polynomial(short_type nn, short_type dd) : std::vector<T>(alpha(nn,dd))
-      { n = nn; d = dd; std::fill(this->begin(), this->end(), T(0)); }
+    polynomial(short_type nn, short_type dd);
     /// Constructor.
-    polynomial(short_type nn, short_type dd, short_type k)
-      : std::vector<T>(alpha(nn,dd)) {
-      n = nn; d = std::max(short_type(1), dd);
-      std::fill(this->begin(), this->end(), T(0));
-      (*this)[k+1] = T(1);
-    }
-    
+    polynomial(short_type nn, short_type dd, short_type k);
   };
+
+  template<typename T> polynomial<T>::polynomial(short_type nn, short_type dd)
+    : std::vector<T>(alpha(nn,dd))
+  { n = nn; d = dd; std::fill(this->begin(), this->end(), T(0)); }
+
+  template<typename T> polynomial<T>::polynomial(short_type nn,
+						 short_type dd, short_type k)
+    : std::vector<T>(alpha(nn,dd)) {
+    n = nn; d = std::max(short_type(1), dd);
+    std::fill(this->begin(), this->end(), T(0));
+    (*this)[k+1] = T(1);
+  }
   
-  
-  template<class T> short_type polynomial<T>::real_degree(void) const {
+  template<typename T> short_type polynomial<T>::real_degree(void) const {
     const_iterator it = this->end() - 1, ite = this->begin() - 1;
     size_type l = this->size();
     for ( ; it != ite; --it, --l) { if (*it != T(0)) break; }
@@ -231,13 +247,13 @@ namespace bgeot
     return dd;
   }
   
-  template<class T> void polynomial<T>::change_degree(short_type dd) {
-    resize(alpha(n,dd));
+  template<typename T> void polynomial<T>::change_degree(short_type dd) {
+    this->resize(alpha(n,dd));
     if (dd > d) std::fill(this->begin() + alpha(n,d), this->end(), T(0));
     d = dd;
   }
   
-  template<class T>
+  template<typename T>
   void polynomial<T>::add_monomial(const T &coeff, const power_index &power) {
     size_type i = power.global_index();
     if (n != power.size()) DAL_THROW(dimension_error, "dimensions mismatch");
@@ -245,7 +261,7 @@ namespace bgeot
     ((*this)[i]) += coeff;
   }
   
-  template<class T> 
+  template<typename T> 
   polynomial<T> &polynomial<T>::operator +=(const polynomial &Q) {
     if (Q.dim() != dim())
       DAL_THROW(dimension_error, "dimensions mismatch");
@@ -257,7 +273,7 @@ namespace bgeot
     return *this;
   }
   
-  template<class T> 
+  template<typename T> 
   polynomial<T> &polynomial<T>::operator -=(const polynomial &Q) {
     if (Q.dim() != dim() || dim() == 0)
       DAL_THROW(dimension_error, "dimensions mismatch");
@@ -269,7 +285,7 @@ namespace bgeot
     return *this;
   }
   
-  template<class T> 
+  template<typename T> 
   polynomial<T> polynomial<T>::operator -(void) const {
     polynomial<T> Q = *this;
     iterator itq = Q.begin(), ite = Q.end();
@@ -277,7 +293,7 @@ namespace bgeot
     return Q;
   }
   
-  template<class T>
+  template<typename T>
   polynomial<T> &polynomial<T>::operator *=(const polynomial &Q) {
     if (Q.dim() != dim())
       DAL_THROW(dimension_error, "dimensions mismatch");
@@ -310,7 +326,7 @@ namespace bgeot
     return *this;
   }
 
-  template<class T>
+  template<typename T>
     void polynomial<T>::direct_product(const polynomial &Q) { 
     polynomial aux = *this;
 
@@ -334,57 +350,56 @@ namespace bgeot
       }
   }
 
-  template<class T>
+  template<typename T>
     polynomial<T> &polynomial<T>::operator *=(const T &e) {
     iterator it = this->begin(), ite = this->end();
     for ( ; it != ite; ++it) (*it) *= e;
     return *this;
   }
 
-  template<class T>
+  template<typename T>
     polynomial<T> &polynomial<T>::operator /=(const T &e) {
     iterator it = this->begin(), ite = this->end();
     for ( ; it != ite; ++it) (*it) /= e;
     return *this;
   }
 
-  template<class T>
+  template<typename T>
     void polynomial<T>::derivative(short_type k) {
     if (k >= n)
       DAL_THROW(std::out_of_range, "index out of range");
     
      iterator it = this->begin(), ite = this->end();
-     power_index mi(dim());
+     power_index mi(dim()); 
      for ( ; it != ite; ++it, ++mi) {
        if ((*it) != T(0) && mi[k] > 0)
-       { mi[k]--; (*this)[mi.global_index()] = (*it) * T(mi[k] + 1); mi[k]++; }
+         { mi[k]--; (*this)[mi.global_index()] = (*it) * T(mi[k] + 1); mi[k]++; }
        *it = T(0);
      }
      if (d > 0) change_degree(d-1);
   }
 
-  template<class T> template<class ITER>
+  template<typename T> template<typename ITER>
     T polynomial<T>::horner(power_index &mi, short_type k, short_type de,
 			    const ITER &it) const {
     if (k == 0)
       return (*this)[mi.global_index()];
     else {
       T v = (*(it+k-1)), res = T(0);
-      short_type *p = &(mi[k-1]);
-      for (*p = degree() - de; *p != short_type(-1); (*p)--)
-	res = horner(mi, k-1, de + *p, it) + v * res;
-      *p = 0;
+      for (mi[k-1] = degree() - de; mi[k-1] != short_type(-1); (mi[k-1])--)
+	res = horner(mi, k-1, de + mi[k-1], it) + v * res;
+      mi[k-1] = 0;
       return res;
     }
   }
 
-  template<class T> template<class ITER>
+  template<typename T> template<typename ITER>
     T polynomial<T>::eval(const ITER &it) const {
     power_index mi(dim());
     return horner(mi, dim(), 0, it);
   }
 
-  template<class ITER>
+  template<typename ITER>
     typename std::iterator_traits<ITER>::value_type
         eval_monomial(const power_index &mi, ITER it) {
     typename std::iterator_traits<ITER>::value_type res
@@ -398,15 +413,13 @@ namespace bgeot
 
 
   /// Print P to the output stream o. for instance cout $<<$ P;
-  template<class T>  std::ostream &operator <<(std::ostream &o,
+  template<typename T>  std::ostream &operator <<(std::ostream &o,
 						     const polynomial<T>& P) { 
     bool first = true; size_type n = 0;
     typename polynomial<T>::const_iterator it = P.begin(), ite = P.end();
     power_index mi(P.dim());
-    
     if (it != ite && *it != T(0))
       { o << *it; first = false; ++it; ++n; ++mi; }
-    
     for ( ; it != ite ; ++it, ++mi ) {
       if (*it != T(0)) {
 	if (!first) { if (*it < T(0)) o << " - "; else o << " + "; }
@@ -414,8 +427,9 @@ namespace bgeot
 	if (dal::abs(*it)!=T(1)) o << dal::abs(*it);
 	for (short_type j = 0; j < P.dim(); ++j)
 	  if (mi[j] != 0) {
-	    o << "x"; if (P.dim() > 1) o << "_" << j; 
-	    if (mi[j]>1) o << "^" << mi[j];
+            if (P.dim() <= 3) o << "xyz"[j];
+            else o << "x_" << j; 
+	    if (mi[j] > 1) o << "^" << mi[j];
 	  }
 	first = false; ++n;
       }
@@ -423,6 +437,34 @@ namespace bgeot
     if (n == 0) o << "0";
     return o;
   }
+
+  /**
+     polynomial variable substitution
+     @param P the original polynomial
+     @param S the substitution poly (not a multivariate one)
+     @param subs_dim : which variable is substituted
+     example: poly_subs(x+y*x^2, x+1, 0) = x+1 + y*(x+1)^2
+  */
+  template<typename T>    
+  polynomial<T> poly_substitute_var(const polynomial<T>& P, const polynomial<T>& S, size_type subs_dim) {
+    if (S.dim()!=1 || subs_dim >= P.dim()) DAL_THROW(failure_error, "wrong arguments for polynomial substitution");
+    polynomial<T> res(P.dim(),0);
+    bgeot::power_index pi(P.dim());
+    std::vector< polynomial<T> > Spow(1);
+    Spow[0] = polynomial<T>(1, 0); Spow[0].one(); // Spow stores powers of S
+    for (size_type k=0; k < P.size(); ++k, ++pi) {
+      if (P[k] == T(0)) continue;
+      while (pi[subs_dim] >= Spow.size()) 
+	Spow.push_back(S*Spow.back());
+      const polynomial<T>& p = Spow[pi[subs_dim]];
+      bgeot::power_index pi2(pi);
+      for (size_type i=0; i < p.size(); ++i) {
+	pi2[subs_dim] = i;
+	res.add_monomial(p[i]*P[k],pi2);
+      }
+    }
+    return res;
+  }
   
   typedef polynomial<opt_long_scalar_type> base_poly;
 
@@ -440,4 +482,4 @@ namespace bgeot
 }  /* end of namespace bgeot.                                           */
 
 
-#endif  /* __BGEOT_POLY_H */
+#endif  /* BGEOT_POLY_H__ */
