@@ -31,8 +31,8 @@
 /* *********************************************************************** */
 
 
-#ifndef __DAL_ALGOBASE_H
-#define __DAL_ALGOBASE_H
+#ifndef DAL_ALGOBASE_H__
+#define DAL_ALGOBASE_H__
 #include <dal_std.h>
 #include <dal_except.h>
 #include <functional>
@@ -172,8 +172,8 @@ namespace dal
   /* ********************************************************************* */
 
   template <class ITER1, class SIZE, class ITER2>
-    void copy_n(ITER1 first, SIZE count, ITER2 result)
-  { for ( ; count > 0; --count) { *result = *first; ++first; ++result; } }
+    ITER2 copy_n(ITER1 first, SIZE count, ITER2 result)
+  { for ( ; count > 0; --count) { *result = *first; ++first; ++result; } return result; }
 
   template<class ITER>
     typename std::iterator_traits<ITER>::value_type
@@ -207,4 +207,4 @@ namespace dal
 
 
 
-#endif /* __DAL_ALGOBASE_H */
+#endif /* DAL_ALGOBASE_H__ */
