@@ -31,8 +31,8 @@
 /*                                                                         */
 /* *********************************************************************** */
 
-#ifndef __DAL_EXCEPT_H
-#define __DAL_EXCEPT_H
+#ifndef DAL_EXCEPT_H__
+#define DAL_EXCEPT_H__
 
 #include <dal_std.h>
 
@@ -133,6 +133,7 @@ namespace dal {
 //     exit(1);
 //   } 
 
+  /* callback handler for gmm/getfem exceptions */
   struct exception_callback {
     virtual void callback(const std::string&) = 0; //{};
 
@@ -150,20 +151,33 @@ namespace dal {
 
   };
 
+  /* crashing callback for debug mode */
+  struct exception_callback_debug : public dal::exception_callback  {
+    virtual void callback(const std::string& msg)
+    { cerr << msg << endl; *(int *)(0) = 0; }
+  };
+
+  /** user function for changing the default exception callback */ 
   inline void set_exception_callback(exception_callback *e)
   { exception_callback::which_except(e); }
 
+#ifdef GETFEM_HAVE_PRETTY_FUNCTION
+#  define DAL_PRETTY_FUNCTION __PRETTY_FUNCTION__
+#else 
+#  define DAL_PRETTY_FUNCTION ""
+#endif
+
 #define DAL_THROW(type, thestr) {                                    \
     std::stringstream msg;                                           \
     msg << "Error in "__FILE__ << ", line "                          \
-        << __LINE__ << ": \n" << thestr << ends;                     \
+        << __LINE__ << " " << DAL_PRETTY_FUNCTION << ": \n" << thestr << ends; \
     dal::exception_callback::do_exception_callback(msg.str());       \
     throw (type)(msg.str());                                         \
   }
 
 #ifdef DEBUG_MODE
 #  define DAL_INTERNAL_ERROR(thestr) { \
-   cerr << "Internal error: " << thestr << endl; \
+  cerr << "Internal error: " << DAL_PRETTY_FUNCTION << " " << thestr << endl; \
    ::abort(); \
    }
 #else
@@ -173,7 +187,7 @@ namespace dal {
 
   struct warning_level {
     static int level(int l = -2)
-    { static int _level = 3; return (l != -2) ? (_level = l) : _level; }
+    { static int level_ = 3; return (l != -2) ? (level_ = l) : level_; }
   };
 
   inline void set_warning_level(int l) { warning_level::level(std::max(0,l)); }
@@ -181,14 +195,14 @@ namespace dal {
 #define DAL_WARNING(level_, thestr) {                                 \
     std::stringstream msg;                                            \
     msg << "Level " << level_ << " Warning in "__FILE__ << ", line "  \
-        << __LINE__ << ": \n" << thestr << ends;                      \
+        << __LINE__ << " " << DAL_PRETTY_FUNCTION << ": \n" << thestr << ends; \
     if ((level_) <= dal::warning_level::level())                      \
        std::cerr << msg.str() << std::endl;                           \
   } 
 
   // Warning levels : 0 always printed
-  //                  1 very important
-  //                  2 important
+  //                  1 very important : specify a possible error in the code.
+  //                  2 important : specify a default of optimization for inst.
   //                  3 remark
 
   
@@ -196,4 +210,4 @@ namespace dal {
 
 
 
-#endif /* __DAL_EXCEPT_H */
+#endif /* DAL_EXCEPT_H__ */
