@@ -31,8 +31,8 @@
 /* *********************************************************************** */
 
 
-#ifndef __DAL_TREE_SORTED_H
-#define __DAL_TREE_SORTED_H
+#ifndef DAL_TREE_SORTED_H__
+#define DAL_TREE_SORTED_H__
 
 #include <dal_tas.h>
 
@@ -46,13 +46,13 @@ namespace dal
   /* d'insertion ou de suppression.                                        */
   /* ********************************************************************* */
 
-  static const size_t _DEPTHMAX_ = size_t(CHAR_BIT*sizeof(size_t)*3) / 2;
+  static const size_t DEPTHMAX__ = size_t(CHAR_BIT*sizeof(size_t)*3) / 2;
   static const size_t ST_NIL = size_t(-1);
 
-  template<class T, class COMP = dal::less<T>, int pks = 5>
+  template<typename T, typename COMP = dal::less<T>, int pks = 5>
     class dynamic_tree_sorted;
 
-  template<class T, class COMP, int pks> struct tsa_iterator
+  template<typename T, typename COMP, int pks> struct tsa_iterator
   {
     typedef T                value_type;
     typedef value_type&      reference;
@@ -63,8 +63,8 @@ namespace dal
     typedef int8_type short_type;
 
     dynamic_tree_sorted<T, COMP, pks> *p;
-    size_type path[_DEPTHMAX_];
-    short_type dir[_DEPTHMAX_];
+    size_type path[DEPTHMAX__];
+    short_type dir[DEPTHMAX__];
     size_type depth;
 
     tsa_iterator(void) {}
@@ -79,7 +79,7 @@ namespace dal
     { return (depth==0) ? ST_NIL : path[depth-1];}
     inline size_type father(void) const
     { return (depth<=1) ? ST_NIL : path[depth-2];}
-    inline size_type _index(void) const { return path[depth-1]; }
+    inline size_type index_(void) const { return path[depth-1]; }
     inline short_type direction(void) const
     { return (depth==0) ? 0 : dir[depth-1];}
     inline void up(void) { if (depth > 0) depth--; }
@@ -103,13 +103,13 @@ namespace dal
     pointer operator->() const { return &(operator*()); }
     
     bool operator ==(const tsa_iterator &i) const
-    { return ((i.depth == 0 && depth == 0) || (i._index() == _index())); }
+    { return ((i.depth == 0 && depth == 0) || (i.index_() == index_())); }
     bool operator !=(const tsa_iterator &i) const
-    { return !((i.depth == 0 && depth == 0) || (i._index() == _index())); }
+    { return !((i.depth == 0 && depth == 0) || (i.index_() == index_())); }
 
   };
 
-  template<class T, class COMP, int pks> 
+  template<typename T, typename COMP, int pks> 
     void tsa_iterator<T, COMP, pks>::copy(const tsa_iterator<T, COMP, pks> &it)
   {
     p = it.p; depth = it.depth;
@@ -120,54 +120,54 @@ namespace dal
     while (p1it != pend) { *p1it++ = *p2it++; *d1it++ = *d2it++; }
   }
   
-  template<class T, class COMP, int pks> 
+  template<typename T, typename COMP, int pks> 
     void tsa_iterator<T, COMP, pks>::down_left(void)
   {
-    #ifdef __GETFEM_VERIFY
-      if (depth <= 0 || depth >= _DEPTHMAX_ || index() == ST_NIL)
+    #ifdef GETFEM_VERIFY
+      if (depth <= 0 || depth >= DEPTHMAX__ || index() == ST_NIL)
 	DAL_THROW(internal_error, "internal error");
     #endif
-    path[depth] = p->left_elt(_index()); dir[depth++] = -1;
+    path[depth] = p->left_elt(index_()); dir[depth++] = -1;
   }
 
-  template<class T, class COMP, int pks> 
+  template<typename T, typename COMP, int pks> 
     void tsa_iterator<T, COMP, pks>::down_right(void)
   { 
-    #ifdef __GETFEM_VERIFY
-      if (depth <= 0 || depth >= _DEPTHMAX_ || index() == ST_NIL)
+    #ifdef GETFEM_VERIFY
+      if (depth <= 0 || depth >= DEPTHMAX__ || index() == ST_NIL)
 	DAL_THROW(internal_error, "internal error");
     #endif
-    path[depth] = p->right_elt(_index()); dir[depth++] = 1;
+    path[depth] = p->right_elt(index_()); dir[depth++] = 1;
   }
 
-  template<class T, class COMP, int pks> 
+  template<typename T, typename COMP, int pks> 
     void tsa_iterator<T, COMP, pks>::down_left_all(void)
-  { while (_index() != ST_NIL) down_left(); up(); }
+  { while (index_() != ST_NIL) down_left(); up(); }
 
-  template<class T, class COMP, int pks> 
+  template<typename T, typename COMP, int pks> 
     void tsa_iterator<T, COMP, pks>::down_right_all(void)
-  { while (_index() != ST_NIL) down_right(); up();}
+  { while (index_() != ST_NIL) down_right(); up();}
    
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     tsa_iterator<T, COMP, pks> &tsa_iterator<T, COMP, pks>::operator ++()
   { 
     if (depth == 0) first();
-    if (p->right_elt(_index()) != ST_NIL) { down_right(); down_left_all(); }
+    if (p->right_elt(index_()) != ST_NIL) { down_right(); down_left_all(); }
     else                { up(); while (dir[depth] == 1) up(); }
     return *this;
   }
   
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     tsa_iterator<T, COMP, pks> &tsa_iterator<T, COMP, pks>::operator --()
   { 
     if (depth == 0) last();
-    if (p->left_elt(_index()) != ST_NIL) { down_left(); down_right_all(); }
+    if (p->left_elt(index_()) != ST_NIL) { down_left(); down_right_all(); }
     else               { up(); while (dir[depth] == -1) up(); }
     return *this;
   }
 
 
-  template<class T, class COMP, int pks> struct const_tsa_iterator
+  template<typename T, typename COMP, int pks> struct const_tsa_iterator
   {
     typedef T                  value_type;
     typedef const value_type&  reference;
@@ -178,8 +178,8 @@ namespace dal
     typedef int8_type short_type;
 
     const dynamic_tree_sorted<T, COMP, pks> *p;
-    size_type path[_DEPTHMAX_];
-    short_type dir[_DEPTHMAX_];
+    size_type path[DEPTHMAX__];
+    short_type dir[DEPTHMAX__];
     size_type depth;
 
     const_tsa_iterator(void) {}
@@ -195,7 +195,7 @@ namespace dal
     { return (depth==0) ? ST_NIL : path[depth-1];}
     inline size_type father(void) const
     { return (depth<=1) ? ST_NIL : path[depth-2];}
-    inline size_type _index(void) const { return path[depth-1]; }
+    inline size_type index_(void) const { return path[depth-1]; }
     inline short_type direction(void) const
     { return (depth==0) ? 0 : dir[depth-1];}
     inline void up(void) { if (depth > 0) depth--; }
@@ -219,13 +219,23 @@ namespace dal
     pointer operator->() const { return &(operator*()); }
     
     bool operator ==(const const_tsa_iterator &i) const
-    { return ((i.depth == 0 && depth == 0) || (i._index() == _index())); }
+    { return ((i.depth == 0 && depth == 0) || (i.index_() == index_())); }
     bool operator !=(const const_tsa_iterator &i) const
-    { return !((i.depth == 0 && depth == 0) || (i._index() == _index())); }
+    { return !((i.depth == 0 && depth == 0) || (i.index_() == index_())); }
 
   };
 
-  template<class T, class COMP, int pks> 
+  template<typename T, typename COMP, int pks>
+  std::ostream& operator<<(std::ostream& o,
+			   const const_tsa_iterator<T,COMP,pks>& it) {
+    o << "const_tsa_iterator : depth=" << it.depth << ", path/dir=[";
+    for (unsigned i=0; i < it.depth; ++i)
+      o << "{" << it.path[i] << ", " << int(it.dir[i]) << "} ";
+    o << "]";
+    return o;
+  }
+
+  template<typename T, typename COMP, int pks> 
     void const_tsa_iterator<T, COMP, pks>::copy(
 				   const const_tsa_iterator<T, COMP, pks> &it)
   {
@@ -237,7 +247,7 @@ namespace dal
     while (p1it != pend) { *p1it++ = *p2it++; *d1it++ = *d2it++; }
   }
 
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     const_tsa_iterator<T, COMP, pks>::const_tsa_iterator(
 					const tsa_iterator<T, COMP, pks> &it)
   {
@@ -249,50 +259,50 @@ namespace dal
     while (p1it != pend) { *p1it++ = *p2it++; *d1it++ = *d2it++; }
   }
 
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     void const_tsa_iterator<T, COMP, pks>::down_left(void)
   {
-    #ifdef __GETFEM_VERIFY
-      if (depth <= 0 || depth >= _DEPTHMAX_ || index() == ST_NIL)
+    #ifdef GETFEM_VERIFY
+      if (depth <= 0 || depth >= DEPTHMAX__ || index() == ST_NIL)
 	DAL_THROW(internal_error, "internal error");
     #endif
-    path[depth] = p->left_elt(_index()); dir[depth++] = -1;
+    path[depth] = p->left_elt(index_()); dir[depth++] = -1;
   }
 
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     void const_tsa_iterator<T, COMP, pks>::down_right(void)
   { 
-    #ifdef __GETFEM_VERIFY
-      if (depth <= 0 || depth >= _DEPTHMAX_ || index() == ST_NIL)
+    #ifdef GETFEM_VERIFY
+      if (depth <= 0 || depth >= DEPTHMAX__ || index() == ST_NIL)
 	DAL_THROW(internal_error, "internal error");
     #endif
-    path[depth] = p->right_elt(_index()); dir[depth++] = 1;
+    path[depth] = p->right_elt(index_()); dir[depth++] = 1;
   }
 
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     void const_tsa_iterator<T, COMP, pks>::down_left_all(void)
-  { while (_index() != ST_NIL) down_left(); up(); }
+  { while (index_() != ST_NIL) down_left(); up(); }
 
-  template<class T, class COMP, int pks> 
+  template<typename T, typename COMP, int pks> 
     void const_tsa_iterator<T, COMP, pks>::down_right_all(void) 
-  { while (_index() != ST_NIL) down_right(); up();}
+  { while (index_() != ST_NIL) down_right(); up();}
   
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     const_tsa_iterator<T, COMP, pks> &
       const_tsa_iterator<T, COMP, pks>::operator ++()
   {  
     if (depth == 0) last();
-    if (p->right_elt(_index()) != ST_NIL) { down_right(); down_left_all(); }
+    if (p->right_elt(index_()) != ST_NIL) { down_right(); down_left_all(); }
     else                { up(); while (dir[depth] == 1) up(); }
     return *this;
   }
 
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     const_tsa_iterator<T, COMP, pks> &
       const_tsa_iterator<T, COMP, pks>::operator --()
   {  
     if (depth == 0) last();
-    if (p->left_elt(_index()) != ST_NIL) { down_left(); down_right_all(); }
+    if (p->left_elt(index_()) != ST_NIL) { down_left(); down_right_all(); }
     else               { up(); while (dir[depth] == -1) up(); }
     return *this;
   }
@@ -301,7 +311,7 @@ namespace dal
   /* Definitition of dynamic_tree_sorted.                                  */
   /* ********************************************************************* */
 
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     class dynamic_tree_sorted : public dynamic_tas<T, pks>
   {
     public :
@@ -344,7 +354,7 @@ namespace dal
 
     public :
 
-      #ifdef __GETFEM_VERIFY
+      #ifdef GETFEM_VERIFY
 
         int verify_balance(size_type i = size_type(-2)) const
         {
@@ -419,7 +429,7 @@ namespace dal
       const_sorted_iterator sorted_ge(const T &elt) const;
   }; 
 
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     std::ostream& operator <<(std::ostream& o,
 			      dynamic_tree_sorted<T, COMP, pks> &m)
   {
@@ -431,7 +441,7 @@ namespace dal
     return o;
   }
 
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     typename dynamic_tree_sorted<T, COMP, pks>::size_type
       dynamic_tree_sorted<T, COMP, pks>::rotate_right(size_type i)
   {
@@ -442,7 +452,7 @@ namespace dal
     return f;
   }
 
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     typename dynamic_tree_sorted<T, COMP, pks>::size_type
       dynamic_tree_sorted<T, COMP, pks>::rotate_left(size_type i)
   {
@@ -453,7 +463,7 @@ namespace dal
     return f;
   }
 
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     typename dynamic_tree_sorted<T, COMP, pks>::size_type
       dynamic_tree_sorted<T, COMP, pks>::rotate_left_right(size_type i)
   {
@@ -475,7 +485,7 @@ namespace dal
     return f;
   }
 
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     typename dynamic_tree_sorted<T, COMP, pks>::size_type
       dynamic_tree_sorted<T, COMP, pks>::rotate_right_left(size_type i)
   { 
@@ -494,7 +504,7 @@ namespace dal
     return f;
   }
 
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     typename dynamic_tree_sorted<T, COMP, pks>::size_type
       dynamic_tree_sorted<T, COMP, pks>::balance_again(size_type i)
   {
@@ -506,16 +516,16 @@ namespace dal
       case +2 : if (nodes[pn->r].eq == 1) return rotate_left(i);
                                       else return rotate_right_left(i);
       case  0 : case -1 : case 1 : return i;
-    #ifdef __GETFEM_VERIFY
+    #ifdef GETFEM_VERIFY
       default : DAL_THROW(internal_error, "internal error");
     #endif
     }
-    #ifndef __GETFEM_VERIFY
+    #ifndef GETFEM_VERIFY
       return ST_NIL;
     #endif
   }
 
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
    void dynamic_tree_sorted<T, COMP, pks>::search_sorted_iterator(const T &elt,
 					     const_sorted_iterator &it) const
   {
@@ -528,7 +538,7 @@ namespace dal
     }
   }
 
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     void dynamic_tree_sorted<T, COMP, pks>::find_sorted_iterator(size_type i,
 						    const_sorted_iterator &it) const 
   {
@@ -546,7 +556,7 @@ namespace dal
     /* pour eviter de faire tout le tableau en cas de faux indice.         */
   }
 
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     void dynamic_tree_sorted<T, COMP, pks>::insert_path(const T &elt,
 					 const_sorted_iterator &it) const
   {
@@ -558,7 +568,7 @@ namespace dal
     }
   }
 
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     typename dynamic_tree_sorted<T, COMP, pks>::size_type
       dynamic_tree_sorted<T, COMP, pks>::search_ge(const T &elt) const 
   {
@@ -569,30 +579,29 @@ namespace dal
     return it.index();
   }
 
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     typename dynamic_tree_sorted<T, COMP, pks>::sorted_iterator
       dynamic_tree_sorted<T, COMP, pks>::sorted_ge(const T &elt)
   {
-    const_sorted_iterator it(*this); insert_path(elt, it);
+    sorted_iterator it(*this); insert_path(elt, it);
     short_type dir = it.direction();
-    if (it.index() == ST_NIL)
-    { it.up(); if (it.index() != ST_NIL && dir == +1) ++it; }
+    it.up(); if (it.index() != ST_NIL && dir == +1) ++it;
     return it;
   } 
 
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     typename dynamic_tree_sorted<T, COMP, pks>::const_sorted_iterator
       dynamic_tree_sorted<T, COMP, pks>::sorted_ge(const T &elt) const
   {
     const_sorted_iterator it(*this); insert_path(elt, it);
     short_type dir = it.direction();
-    if (it.index() == ST_NIL)
-    { it.up(); if (it.index() != ST_NIL && dir == +1) ++it; }
+    it.up(); 
+    if (it.index() != ST_NIL && dir == +1) ++it;
     return it;
   }
   
 
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     typename dynamic_tree_sorted<T, COMP, pks>::size_type
       dynamic_tree_sorted<T, COMP, pks>::memsize(void) const
   {
@@ -600,7 +609,7 @@ namespace dal
       + sizeof(dynamic_tree_sorted<T, COMP, pks>);
   }
 
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     void dynamic_tree_sorted<T, COMP, pks>::compact(void)
   { 
     if (!this->empty())
@@ -608,7 +617,7 @@ namespace dal
 	swap(this->ind.first_false(), this->ind.last_true());
   }
 
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     void dynamic_tree_sorted<T, COMP, pks>::add_index(size_type i,
 						      const_sorted_iterator &it)
   {
@@ -645,7 +654,7 @@ namespace dal
     }
   }
 
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     typename dynamic_tree_sorted<T, COMP, pks>::size_type
       dynamic_tree_sorted<T, COMP, pks>::add(const T &f)
   {
@@ -655,7 +664,7 @@ namespace dal
     return num;
   }
 
-  template<class T, class COMP, int pks> void
+  template<typename T, typename COMP, int pks> void
       dynamic_tree_sorted<T, COMP, pks>::add_to_index(size_type i,const T &f) {
     if (!(this->index_valid(i) && compar(f, (*this)[i]) == 0)) {
       if (this->index_valid(i)) sup(i);
@@ -665,7 +674,7 @@ namespace dal
     }
   }
 
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     typename dynamic_tree_sorted<T, COMP, pks>::size_type
       dynamic_tree_sorted<T, COMP, pks>::add_norepeat(const T &f,
 						  bool replace, bool *present)
@@ -685,7 +694,7 @@ namespace dal
     return num;
   }  
 
-  template<class T, class COMP, int pks> 
+  template<typename T, typename COMP, int pks> 
     void dynamic_tree_sorted<T, COMP, pks>::resort(void) 
   {
     const_tas_iterator itb
@@ -698,7 +707,7 @@ namespace dal
     { insert_path(*itb, it); add_index(itb.index(), it); ++itb; }
   }     
 
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     void dynamic_tree_sorted<T, COMP, pks>::sup_index(size_type i,
 						      const_sorted_iterator &it)
   {
@@ -755,7 +764,7 @@ namespace dal
     }
   }
   
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     void dynamic_tree_sorted<T, COMP, pks>::sup(size_type i)
   {
     if (i >= INT_MAX)
@@ -766,7 +775,7 @@ namespace dal
     { sup_index(i, it); dynamic_tas<T, pks>::sup(i); }
   }
 
-  template<class T, class COMP, int pks>
+  template<typename T, typename COMP, int pks>
     void dynamic_tree_sorted<T, COMP, pks>::swap(size_type i, size_type j)
   {
     if (i >= INT_MAX || j >= INT_MAX) {
@@ -799,7 +808,7 @@ namespace dal
   /* ********************************************************************* */
   /* pas completement satisfaisant. A utiliser avec precautions.           */
 
-  template<class T, class TAB, class COMP> struct less_index
+  template<typename T, typename TAB, typename COMP> struct less_index
        : public std::binary_function<size_t, size_t, int>
   {
     const TAB *tab;
@@ -819,7 +828,7 @@ namespace dal
   };
 
 
-  template<class T, class TAB, class COMP = dal::less<T>, int pks = 5>
+  template<typename T, typename TAB, typename COMP = dal::less<T>, int pks = 5>
     class dynamic_tree_sorted_index : public
          dynamic_tree_sorted<size_t, dal::less_index<T,TAB,COMP>, pks>
   {
@@ -861,4 +870,4 @@ namespace dal
 
 }
 
-#endif /* __DAL_TREE_SORTED_H */
+#endif /* DAL_TREE_SORTED_H__ */
