@@ -42,12 +42,12 @@ namespace ftool
   /* ********************************************************************* */
 
 
-  bool read_untill(std::istream &ist, const char *st);
+  bool read_until(std::istream &ist, const char *st);
   bool get_token(std::istream &ist, char *st, int nb);
 
   struct skip {
     const char *s;
-    skip(const char *_s) : s(_s) {}
+    skip(const char *s_) : s(s_) {}
   };
   std::istream& operator>>(std::istream& is, const skip& t);
 
@@ -69,7 +69,16 @@ namespace ftool
   /*       Clock functions.                                                */
   /* ********************************************************************* */
 
-  double uclock_sec(void);
+# ifdef HAVE_SYS_TIMES
+  inline double uclock_sec(void) {
+    static double ttclk = 0.;
+    if (ttclk == 0.) ttclk = sysconf(_SC_CLK_TCK);
+    tms t; times(&t); return double(t.tms_utime) / ttclk;
+  }
+# else
+  inline double uclock_sec(void)
+  { return double(clock())/double(CLOCKS_PER_SEC); }
+# endif
 
   /* ********************************************************************* */
   /*       Read a parameter file.                                          */
