@@ -31,16 +31,14 @@
 /* *********************************************************************** */
 
 
-#ifndef __GETFEM_ASSEMBLING_H
-#define __GETFEM_ASSEMBLING_H
+#ifndef GETFEM_ASSEMBLING_H__
+#define GETFEM_ASSEMBLING_H__
 
-#include <getfem_mesh_fem.h>
-#include <gmm.h>
 #include <getfem_assembling_tensors.h>
 namespace getfem
 {
   /**
-     compute $\|U\|_2$
+     compute $\|U\|2_$
   */
   template<class VEC>
   scalar_type asm_L2_norm(const mesh_fem &mf, const VEC &U) {
@@ -63,7 +61,7 @@ namespace getfem
   }
 
   /**
-     compute $\|\nabla U\|_2$
+     compute $\|\nabla U\|2_$
    */
   template<class VEC>
   scalar_type asm_H1_semi_norm(const mesh_fem &mf, const VEC &U) {
@@ -139,7 +137,7 @@ namespace getfem
   */
   template<class VECT1, class VECT2>
     void asm_source_term(VECT1 &B, const mesh_fem &mf,
-			 const mesh_fem &mfdata, VECT2 &F, size_type boundary=size_type(-1))
+			 const mesh_fem &mfdata, const VECT2 &F, size_type boundary=size_type(-1))
   {
     generic_assembly assem;
     if (mf.get_qdim() == 1)
@@ -173,19 +171,34 @@ namespace getfem
     (fortran style)
 
     Works for both volumic assembly and boundary assembly
-   */
+  */
   template<class MAT, class VECT>
     void asm_qu_term(MAT &M, 
 		     const mesh_fem &mf_u, 
-		     const mesh_fem &mf_d, const VECT &Q, size_type boundary=size_type(-1))
+		     const mesh_fem &mf_d, const VECT &Q, 
+		     size_type boundary=size_type(-1))
   {
     generic_assembly assem;
     if (mf_u.get_qdim() == 1)
       assem.set("Q=data$1(#2);"
 		"M(#1,#1)+=comp(Base(#1).Base(#1).Base(#2))(:,:,k).Q(k);");
-    else
-      assem.set("Q=data$1(qdim(#1),qdim(#1),#2);"
-		"M(#1,#1)+=comp(vBase(#1).vBase(#1).Base(#2))(:,i,:,j,k).Q(i,j,k);");
+    else {
+      /* detect the symmetricity of Q (in that case the symmetricity of the final
+	 matrix will be ensured, and computations will be slightly speed up */
+      bool Q_symmetric = true;
+      size_type q = mf_u.get_qdim();
+      for (size_type k=0; k < mf_d.nb_dof(); ++k)
+	for (size_type i=1; i < q; ++i)
+	  for (size_type j=0; j < i; ++j)
+	    if (Q[k*q*q+i*q+j] != Q[k*q*q+j*q+i]) { Q_symmetric = false; goto bye; }
+    bye:
+      if (Q_symmetric)
+	assem.set("Q=data$1(qdim(#1),qdim(#1),#2);"
+		  "M(#1,#1)+=sym(comp(vBase(#1).vBase(#1).Base(#2))(:,i,:,j,k).Q(i,j,k));");
+      else
+	assem.set("Q=data$1(qdim(#1),qdim(#1),#2);"
+		  "M(#1,#1)+=comp(vBase(#1).vBase(#1).Base(#2))(:,i,:,j,k).Q(i,j,k);");
+    }
     assem.push_mf(mf_u);
     assem.push_mf(mf_d);
     assem.push_data(Q);
@@ -202,10 +215,11 @@ namespace getfem
   */
   template<class MAT, class VECT>
     void asm_stiffness_matrix_for_linear_elasticity(MAT &RM,
-						   mesh_fem &mf, 
-						   mesh_fem &mfdata, 
-						   VECT &LAMBDA, VECT &MU)
+						   const mesh_fem &mf, 
+						   const mesh_fem &mfdata, 
+						   const VECT &LAMBDA, const VECT &MU)
   {
+    if (mf.get_qdim() != mf.linked_mesh().dim()) DAL_THROW(std::logic_error, "wrong qdim for the mesh_fem");
     /* e = strain tensor,
        M = 2*mu*e(u):e(v) + lambda*tr(e(u))*tr(e(v))
     */
@@ -227,9 +241,9 @@ namespace getfem
   */
   template<class MAT, class VECT>
     void asm_stiffness_matrix_for_linear_elasticity_Hooke(MAT &RM,
-							 mesh_fem &mf, 
-							 mesh_fem &mfdata, 
-							 VECT &H)
+							  const mesh_fem &mf, 
+							  const mesh_fem &mfdata, 
+							  const VECT &H)
   {
     /* e = strain tensor,
        M = a_{i,j,k,l}e_{i,j}(u)e_{k,l}(v)
@@ -253,13 +267,13 @@ namespace getfem
     void asm_stokes(MAT &K, MAT &B, 
 		    const mesh_fem &mf_u,
 		    const mesh_fem &mf_p,
-		    const mesh_fem &mf_d, VECT &viscos)
+		    const mesh_fem &mf_d, const VECT &viscos)
   {
     generic_assembly assem("visc=data$1(#3); "
 			   "t=comp(vGrad(#1).vGrad(#1).Base(#3));"
 			   "e=(t{:,2,3,:,5,6,:}+t{:,3,2,:,5,6,:}+t{:,2,3,:,6,5,:}+t{:,3,2,:,6,5,:})/4;"
 			   "M$1(#1,#1) += sym(e(:,i,j,:,i,j,k).visc(k));"          // visc*D(u):D(v)
-			   "M$2(#1,#2) += comp(vGrad(#1).Base(#2))(:,i,i,:);"); // p.div v
+			   "M$2(#1,#2) += comp(vGrad(#1).Base(#2))(:,i,i,:);");    // p.div v
     assem.push_mf(mf_u);
     assem.push_mf(mf_p);
     assem.push_mf(mf_d);
@@ -274,7 +288,7 @@ namespace getfem
   */
   template<class MAT, class VECT>
     void asm_stiffness_matrix_for_laplacian(MAT &M, const mesh_fem &mf,
-					   const mesh_fem &mfdata, VECT &A)
+					   const mesh_fem &mfdata, const VECT &A)
   {
     generic_assembly assem("a=data$1(#2); M$1(#1,#1)+=sym(comp(Grad(#1).Grad(#1).Base(#2))(:,i,:,i,j).a(j))");
     //generic_assembly assem("a=data$1(#2); M$1(#1,#1)+=comp(Grad(#1).Grad(#1).Base(#2))(:,i,:,i,j).a(j)");
@@ -286,12 +300,27 @@ namespace getfem
   }
 
   /**
-     assembly of $\int_\Omega A(x)\nabla u.\nabla v$, where $A(x)$ is a matrix.
+     assembly of $\int_\Omega A(x)\nabla u.\nabla v$, where $A(x)$ is a NxN matrix.
+     Arguments:
+      - M  : a sparse matrix of dimensions mf.nb_dof() x mf.nb_dof()
+
+      - mf : the mesh_fem that describes the solution, with
+      mf.get_qdim() == N.
+
+      - mfdata : the mesh_fem that describes the coefficients of A
+      (mfdata.get_qdim() == 1).
+
+      - A__ : a (very large) vector, which is a flattened (n x n x
+      mfdata.nb_dof()) 3D array. For each dof of mfdata, it contains
+      the n x n coefficients of A. As usual, the order is the
+      "fortran-order", i.e. A__ = [A_11(dof1) A_21(dof1) A_31(dof1)
+      A_12(dof1) A_22(dof1) ... A_33(dof) A_11(dof2)
+      .... A_33(lastdof)]
   */
   template<class MAT, class VECT>
     void asm_stiffness_matrix_for_scalar_elliptic(MAT &M, const mesh_fem &mf,
 						  const mesh_fem &mfdata,
-						  VECT &A)
+						  const VECT &A)
   {
     generic_assembly assem("a=data$1(mdim(#1),mdim(#1),#2); M$1(#1,#1)+=comp(Grad(#1).Grad(#1).Base(#2))(:,i,:,j,k).a(j,i,k)");
     assem.push_mf(mf);
@@ -311,15 +340,14 @@ namespace getfem
   /* ********************************************************************* */
   /*	Stiffness matrix for laplacian.                                     */
   /* ********************************************************************* */
-
+#if 0
   template<class MAT, class VECT>
     void assembling_stiffness_matrix_for_laplacian(MAT &RM, const mesh_fem &mf,
 						  const mesh_fem &mfdata, const VECT &A)
   { // optimisable
 
     DAL_WARNING(3, "obsolete function - use asm_stiffness_matrix_for_laplacian");
-    size_type cv, nbd1, nbd2, N = mf.linked_mesh().dim();
-    dal::bit_vector nn = mf.convex_index();
+    size_type nbd1, nbd2, N = mf.linked_mesh().dim();
     base_tensor t;
     pfem pf1, pf2, pf1prec = 0, pf2prec = 0;
     pintegration_method pim, pimprec = 0;
@@ -330,8 +358,7 @@ namespace getfem
       DAL_THROW(std::invalid_argument,
 		"This assembling procedure only works on a single mesh");
 
-    for (cv << nn; cv != ST_NIL; cv << nn)
-    {
+    for (dal::bv_visitor cv(mf.convex_index()); !cv.finished(); ++cv) {
       pf1 =     mf.fem_of_element(cv); nbd1 = pf1->nb_dof();
       pf2 = mfdata.fem_of_element(cv); nbd2 = pf2->nb_dof();
       pgt = mf.linked_mesh().trans_of_convex(cv);
@@ -343,7 +370,7 @@ namespace getfem
 	pmec = mat_elem(pme, pim, pgt);
 	pf1prec = pf1; pf2prec = pf2; pgtprec = pgt; pimprec = pim;
       }
-      pmec->gen_compute(t, mf.linked_mesh().points_of_convex(cv));
+      pmec->gen_compute(t, mf.linked_mesh().points_of_convex(cv), cv);
       // cout << "elem matrix " << t << endl;
       base_tensor::iterator p = t.begin();
       for (size_type r = 0; r < nbd2; r++) {
@@ -378,8 +405,6 @@ namespace getfem
 					const mesh_fem &mf_d,
 					const VECT &DATA) {
     DAL_WARNING(3, "obsolete function - use asm_stokes");
-    size_type cv;
-    dal::bit_vector nn = mf_u.convex_index();
 
     base_tensor t;
 
@@ -398,8 +423,7 @@ namespace getfem
 		"This assembling procedure only works on a single mesh");
 
     /* loop over all convexes */
-    for (cv << nn; cv != ST_NIL; cv << nn) {
-
+    for (dal::bv_visitor cv(mf_u.convex_index()); !cv.finished(); ++cv) {
       pf_u = mf_u.fem_of_element(cv); nbdof_u = pf_u->nb_dof();
       pf_p = mf_p.fem_of_element(cv); nbdof_p = pf_p->nb_dof();
       pf_d = mf_d.fem_of_element(cv); nbdof_d = pf_d->nb_dof();
@@ -416,7 +440,7 @@ namespace getfem
 	pf_p_prec = pf_p;
 	pf_d_prec = pf_d; pgtprec = pgt; pimprec = pim;
       }
-      pmec->gen_compute(t, mf_u.linked_mesh().points_of_convex(cv));
+      pmec->gen_compute(t, mf_u.linked_mesh().points_of_convex(cv), cv);
       
       base_tensor::iterator p = t.begin();
       for (size_type i = 0; i < nbdof_d; i++) {
@@ -460,8 +484,7 @@ namespace getfem
   { //  verifier
     DAL_WARNING(3, "obsolete function - use asm_stiffness_matrix_for_linear_elasticity");
 
-    size_type cv, nbd1, nbd2, N = mf.linked_mesh().dim();
-    dal::bit_vector nn = mf.convex_index();
+    size_type nbd1, nbd2, N = mf.linked_mesh().dim();
     base_tensor t;
     pfem pf1, pf2, pf1prec = NULL, pf2prec = NULL;
     pintegration_method pim, pimprec = 0;
@@ -472,8 +495,7 @@ namespace getfem
       DAL_THROW(std::invalid_argument,
 		"This assembling procedure only works on a single mesh");
   
-    for (cv << nn; cv != ST_NIL; cv << nn)
-    {
+    for (dal::bv_visitor cv(mf.convex_index()); !cv.finished(); ++cv) {
       pf1 =     mf.fem_of_element(cv); nbd1 = pf1->nb_dof();
       pf2 = mfdata.fem_of_element(cv); nbd2 = pf2->nb_dof();
       pgt = mf.linked_mesh().trans_of_convex(cv);
@@ -486,7 +508,7 @@ namespace getfem
 	pmec = mat_elem(pme, pim, pgt);
 	pf1prec = pf1; pf2prec = pf2; pgtprec = pgt; pimprec = pim;
       }
-      pmec->gen_compute(t, mf.linked_mesh().points_of_convex(cv));
+      pmec->gen_compute(t, mf.linked_mesh().points_of_convex(cv), cv);
       base_tensor::iterator p = t.begin();
       
       size_type nbd = mf.nb_dof_of_element(cv);
@@ -531,98 +553,7 @@ namespace getfem
       if (p != t.end()) DAL_THROW(dal::internal_error, "internal error"); 
     }
   }
-
-
-  template<class MAT, class VECT>
-  void assembling_boundary_qu_term(MAT &M, 
-				   const mesh_fem &mf_u, size_type boundary, 
-				   const mesh_fem &mf_d, const VECT &Q, dim_type N)
-  {
-    DAL_WARNING(3, "obsolete function - use asm_qu_term");
-    size_type cv;
-    dal::bit_vector nn = mf_u.convex_index(), nf;
-    base_tensor t;
-    pfem pf_u, pf_d, pf_u_prec = NULL, pf_d_prec = NULL;
-    pintegration_method pim, pimprec = 0;
-    bgeot::pgeometric_trans pgt, pgtprec = NULL;
-    pmat_elem_type pme; pmat_elem_computation pmec = 0;
-
-    if (&(mf_u.linked_mesh()) != &(mf_d.linked_mesh()))
-      DAL_THROW(std::invalid_argument,
-		"This assembling procedure only works on a single mesh");
-
-    for (cv << nn; cv != ST_NIL; cv << nn)
-    {
-      nf = mf_u.faces_of_convex_on_boundary(cv, boundary);
-      if (nf.card() > 0)
-      {
-	size_type f, nbdof_u, nbdof_d;
-
-	pf_u = mf_u.fem_of_element(cv); nbdof_u = pf_u->nb_dof();
-	pf_d = mf_d.fem_of_element(cv); nbdof_d = pf_d->nb_dof();
-	pgt = mf_u.linked_mesh().trans_of_convex(cv);
-	pim = mf_u.int_method_of_element(cv);
-	if (pf_u_prec != pf_u || pf_d_prec != pf_d || pgtprec!=pgt 
-	    || pimprec != pim)
-	{
-	  pme = mat_elem_product(mat_elem_base(pf_d), 
-				 mat_elem_product(mat_elem_base(pf_u),
-						  mat_elem_base(pf_u)));
-	  pmec = mat_elem(pme, pim, pgt);
-	  pf_u_prec = pf_u; pf_d_prec = pf_d; pgtprec = pgt; pimprec = pim;
-	}
-	for (f << nf; f != ST_NIL; f << nf)
-	{
-	  pmec->gen_compute_on_face(t,mf_u.linked_mesh().points_of_convex(cv),
-				    f);
-	  base_tensor::iterator p = t.begin();
-	  scalar_type vmax = gmm::vect_norminf(base_vector(t));
-
-	  for (size_type j = 0; j < nbdof_u; j++) {
-	    size_type dof_j = mf_u.ind_dof_of_element(cv)[j];
-	    for (size_type i = 0; i < nbdof_u; i++) {
-	      size_type dof_i = mf_u.ind_dof_of_element(cv)[i];
-	      for (size_type id = 0; id < nbdof_d; id++) {
-
-		size_type dof_d = mf_d.ind_dof_of_element(cv)[id];
-
-		/* for every element of the matrix Q */
-		for (int ii=0; ii < N; ii++) {
-		  for (int jj=0; jj < N; jj++) {
-		    /* get Q[ii][jj] for the degree of freedom 'dof_d' */
-		    scalar_type data = Q[(jj*N+ii) + N*N*(dof_d)];
-
-		    /* we filter out noise since this matrix can be used 
-		       as a constraints matrix for dirichlet conditions,
-		       noise may lead to 'fictive' dirichlet condition
-		       (this is the case for ex. with laplace/PK(1,4)) 
-
-		       NON NON ET NON !!
-		       finaly we DON'T FILTER NOISE since it breaks 
-		       the assembling of dirichlet conditions against
-		       hierarchical FEMS ...
-		    */
-		    if (data != 0.) {// && vmax != .0 && (*p)/vmax > 1e-5) {
-		      /*
-			cerr << "QU : adding " << data << "*" << (*p) << " at dof_i=" << 
-			dof_i << "*" << N << "+" << ii << ", dof_j=" << dof_i << "*" << 
-			N << "+" << ii << endl;
-		      */
-		      M(dof_i*N+ii, dof_j*N+jj) += data* (*p);
-		    }
-		  }
-		}
-		p++;
-	      }
-	    }
-	  }
-	  if (p != t.end()) DAL_THROW(dal::internal_error, "internal error"); 
-	}
-      }
-    }
-  }
-
-
+#endif
 
   /* 
      new version, which takes into account the qdim dimension of the mesh_fem 
@@ -636,8 +567,7 @@ namespace getfem
   void asm_dirichlet_constraints(MAT &M, VECT &B, const mesh_fem &mf_u,
 				 const mesh_fem &mf_rh,
 				 const VECT &H, const VECT &R, size_type boundary) {
-    size_type cv;
-    dal::bit_vector nn = mf_u.convex_index(), nf;
+    dal::bit_vector nf;
     pfem pf_u, pf_rh;
     
     if (mf_rh.get_qdim() != 1)
@@ -647,7 +577,7 @@ namespace getfem
 
     /* step 2 : simplification of simple dirichlet conditions 
      */
-    for (cv << nn; cv != ST_NIL; cv << nn) {
+    for (dal::bv_visitor cv(mf_u.convex_index()); !cv.finished(); ++cv) {
       nf = mf_u.faces_of_convex_on_boundary(cv, boundary);
       /* don't try anything with vector elements */
       if (mf_u.fem_of_element(cv)->target_dim() != 1) continue;
@@ -677,10 +607,8 @@ namespace getfem
 		with             H_j*phi_i = R_j     
 	      */
 	      if (tdof_u == tdof_rh &&
-		  bgeot::vect_dist2(pf_u->node_convex().points()[ind_u], 
-				    pf_rh->node_convex().points()[ind_rh]) < 1.0E-7) {
-		// to be optimized (square root ..!)
-
+		  bgeot::vect_dist2_sqr(pf_u->node_convex().points()[ind_u], 
+					pf_rh->node_convex().points()[ind_rh]) < 1.0E-14) {
 		/* the dof might be "duplicated" */
 		for (size_type q = 0; q < Q; ++q) {
 		  size_type dof_u = mf_u.ind_dof_of_element(cv)[ind_u*Q + q];
@@ -726,6 +654,7 @@ namespace getfem
   }
 
   /* old version, pre-Qdim */
+#if 0
   template<class MAT, class VECT>
   void assembling_dirichlet_constraints(MAT &M, VECT &B, const mesh_fem &mf_u,
 					size_type boundary,
@@ -733,13 +662,12 @@ namespace getfem
 				        const VECT &H, const VECT &R,
 					dim_type N) {
     DAL_WARNING(3, "obsolete function - use asm_dirichlet_constraints");
-    size_type cv;
-    dal::bit_vector nn = mf_u.convex_index(), nf;
+    dal::bit_vector nf;
     pfem pf_u, pf_rh;
     
     assembling_boundary_qu_term(M, mf_u, boundary, mf_rh, H, N);
     assembling_Neumann_condition(B, mf_u, boundary, mf_rh, R, N);
-    for (cv << nn; cv != ST_NIL; cv << nn) {
+    for (dal::bv_visitor cv(mf_u.convex_index()); !cv.finished(); ++cv) {
       nf = mf_u.faces_of_convex_on_boundary(cv, boundary);
       if (nf.card() > 0) {
 	pf_u = mf_u.fem_of_element(cv); 
@@ -792,8 +720,7 @@ namespace getfem
 		     const MESH_FEM &mf2, dim_type N)
   {
     DAL_WARNING(3, "obsolete function - use asm_mass_matrix");
-    size_type cv, nbd1, nbd2;
-    dal::bit_vector nn = mf1.convex_index();
+    size_type nbd1, nbd2;
     base_tensor t;
     pfem pf1, pf1prec = 0, pf2, pf2prec = 0;
     pintegration_method pim, pimprec = 0;
@@ -805,8 +732,7 @@ namespace getfem
       DAL_THROW(std::invalid_argument,
 		"This assembling procedure only works on a single mesh");
 
-    for (cv << nn; cv != ST_NIL; cv << nn)
-    {
+    for (dal::bv_visitor cv(mf1.convex_index()); !cv.finished(); ++cv) {
       pf1 = mf1.fem_of_element(cv); nbd1 = pf1->nb_dof();
       pf2 = mf2.fem_of_element(cv); nbd2 = pf2->nb_dof();
       pgt = mf1.linked_mesh().trans_of_convex(cv);
@@ -817,7 +743,7 @@ namespace getfem
 	pmec = mat_elem(pme, pim, pgt);
 	pf1prec = pf1; pf2prec = pf2; pgtprec = pgt; pimprec = pim;
       }
-      pmec->gen_compute(t, mf1.linked_mesh().points_of_convex(cv));
+      pmec->gen_compute(t, mf1.linked_mesh().points_of_convex(cv), cv);
 
       // cout << "t = " << t << endl;
       
@@ -847,8 +773,8 @@ namespace getfem
 		    const MESH_FEM &mf2, size_type boundary, dim_type N)
   {
     DAL_WARNING(3, "obsolete function - use asm_mass_matrix");
-    size_type cv, nbd1, nbd2, f;
-    dal::bit_vector nn = mf1.convex_index(), nf;
+    size_type nbd1, nbd2, f;
+    dal::bit_vector nf;
     base_tensor t;
     pfem pf1, pf1prec = 0, pf2, pf2prec = 0;
     pintegration_method pim, pimprec = 0;
@@ -860,8 +786,7 @@ namespace getfem
       DAL_THROW(std::invalid_argument,
 		"This assembling procedure only works on a single mesh");
 
-    for (cv << nn; cv != ST_NIL; cv << nn)
-    {
+    for (dal::bv_visitor cv(mf1.convex_index()); !cv.finished(); ++cv) {
       nf = mf1.faces_of_convex_on_boundary(cv, boundary);
       if (nf.card() > 0) {
 	pf1 = mf1.fem_of_element(cv); nbd1 = pf1->nb_dof();
@@ -878,7 +803,7 @@ namespace getfem
 	for (f << nf; f != ST_NIL; f << nf) {
 
 	  pmec->gen_compute_on_face(t, mf1.linked_mesh().points_of_convex(cv),
-				    f);
+				    f, cv);
 	  
 	  base_tensor::iterator p = t.begin();
 	  for (size_type i = 0; i < nbd2; i++) {
@@ -911,8 +836,7 @@ namespace getfem
 					const mesh_fem &mfdata, const VECT2 &F, dim_type N)
   {
     DAL_WARNING(3, "obsolete function - use asm_source_term");
-    size_type cv, nbd1, nbd2;
-    dal::bit_vector nn = mf.convex_index();
+    size_type nbd1, nbd2;
     base_tensor t;
     pfem pf1, pf2, pf1prec = NULL, pf2prec = NULL;
     pintegration_method pim, pimprec = 0;
@@ -923,8 +847,7 @@ namespace getfem
       DAL_THROW(std::invalid_argument,
 		"This assembling procedure only works on a single mesh");
 
-    for (cv << nn; cv != ST_NIL; cv << nn)
-    {
+    for (dal::bv_visitor cv(mf.convex_index()); !cv.finished(); ++cv) {
       pf1 =     mf.fem_of_element(cv); nbd1 = pf1->nb_dof();
       pf2 = mfdata.fem_of_element(cv); nbd2 = pf2->nb_dof();
       pgt = mf.linked_mesh().trans_of_convex(cv);
@@ -935,7 +858,7 @@ namespace getfem
 	pmec = mat_elem(pme, pim, pgt);
 	pf1prec = pf1; pf2prec = pf2; pgtprec = pgt; pimprec = pim;
       }
-      pmec->gen_compute(t, mf.linked_mesh().points_of_convex(cv));
+      pmec->gen_compute(t, mf.linked_mesh().points_of_convex(cv), cv);
       base_tensor::iterator p = t.begin();
       for (size_type i = 0; i < nbd2; i++)
       {
@@ -953,20 +876,18 @@ namespace getfem
   /* ********************************************************************* */
   /*	Dirichlet Condition.                                               */
   /* ********************************************************************* */
-
+#endif
   template<class MATRM, class VECT1, class VECT2>
     void assembling_Dirichlet_condition(MATRM &RM, VECT1 &B,
 					const mesh_fem &mf,
 					size_type boundary,
 					const VECT2 &F)
   { // Marche uniquement pour des ddl de lagrange.
-    size_type cv, Q=mf.get_qdim();
-    dal::bit_vector nn = mf.convex_index();
+    size_type Q=mf.get_qdim();
     dal::bit_vector nndof = mf.dof_on_boundary(boundary);
     pfem pf1;
-
-    for (cv << nn; cv != ST_NIL; cv << nn)
-    {
+    cerr << "TODO: renommer ou virer assembling_Dirichlet_condition\n";
+    for (dal::bv_visitor cv(mf.convex_index()); !cv.finished(); ++cv) {
       pf1 = mf.fem_of_element(cv);
       pdof_description ldof = lagrange_dof(pf1->dim());
       size_type nbd = pf1->nb_dof();
@@ -1002,21 +923,21 @@ namespace getfem
     void add_Dirichlet_dof(MATRM &RM, VECT1 &B,
 			   const mesh_fem &mf,
 			   size_type dof, scalar_type dof_val) {     
-    size_type cv, Q=mf.get_qdim();
-    dal::bit_vector nn = mf.convex_index();
+    size_type Q=mf.get_qdim();
+    bgeot::mesh_convex_ind_ct dofcv = mf.convex_to_dof(dof);
     pfem pf1;
 
-    // peu performant !
-    for (cv << nn; cv != ST_NIL; cv << nn) {
-      pf1 = mf.fem_of_element(cv);
+    for (bgeot::mesh_convex_ind_ct::const_iterator it = dofcv.begin();
+	 it != dofcv.end(); ++it) {
+      pf1 = mf.fem_of_element(*it);
       if (pf1->target_dim() != 1)
 	DAL_THROW(to_be_done_error, "sorry, to be done ... ");
       size_type nbd = pf1->nb_dof();
       for (size_type i = 0; i < nbd * Q; i++) {
-	size_type dof1 = mf.ind_dof_of_element(cv)[i];
+	size_type dof1 = mf.ind_dof_of_element(*it)[i];
 	if (dof == dof1) {
 	  for (size_type j = 0; j < nbd * Q; j++) {
-	    size_type dof2 = mf.ind_dof_of_element(cv)[j];
+	    size_type dof2 = mf.ind_dof_of_element(*it)[j];
 	    if (!(dof == dof2)) {
 	      B[dof2] -= RM(dof2, dof1) * dof_val;
 	      RM(dof2, dof1) = RM(dof1, dof2) = 0;
@@ -1028,7 +949,7 @@ namespace getfem
     }
   }
 
-
+#if 0
   template<class MATRM, class VECT1, class VECT2>
     void old_assembling_Dirichlet_condition(MATRM &RM, VECT1 &B,
 					const mesh_fem &mf,
@@ -1036,13 +957,10 @@ namespace getfem
 					const VECT2 &F, dim_type N)
   { /* Y-a-il un moyen plus performant ? */
     // Marche uniquement pour des ddl de lagrange.
-    size_type cv;
-    dal::bit_vector nn = mf.convex_index();
     dal::bit_vector nndof = mf.dof_on_boundary(boundary);
     pfem pf1;
 
-    for (cv << nn; cv != ST_NIL; cv << nn)
-    {
+    for (dal::bv_visitor cv(mf.convex_index()); !cv.finished(); ++cv) {
       pf1 = mf.fem_of_element(cv);
       pdof_description ldof = lagrange_dof(pf1->dim());
       size_type nbd = pf1->nb_dof();
@@ -1071,7 +989,7 @@ namespace getfem
       }
     }
   }
-
+#endif
 
   template<class MATD, class MATG, class VECT>
   size_type Dirichlet_nullspace(const MATD &D, MATG &G,
@@ -1089,38 +1007,42 @@ namespace getfem
     // the solution of minimal norm of D*U = UD in UDD and
     // return the dimension of the kernel. The function is based
     // on a Gramm-Schmidt algorithm.
+    typedef typename gmm::linalg_traits<MATD>::value_type T;
+    typedef typename gmm::number_traits<T>::magnitude_type R;
     typedef typename gmm::temporary_vector<MATD>::vector_type TEMP_VECT;
+    R tol = gmm::default_tol(R());
+    R norminfD = gmm::mat_maxnorm(D);
     size_type nbd = gmm::mat_ncols(D), nbase = 0, nbr = gmm::mat_nrows(D);
     TEMP_VECT aux(nbr), e(nbd), f(nbd);
     dal::dynamic_array<TEMP_VECT> base_img;
     dal::dynamic_array<TEMP_VECT> base_img_inv;
     size_type nb_bimg = 0;
-    
+
     if (!(gmm::is_col_matrix(D)))
-      DAL_WARNING(3,
+      DAL_WARNING(2,
 		  "Dirichlet_nullspace is inefficient when D is a row matrix");
     // First, detection of null columns of D, and already orthogonals 
     // vectors of the image of D.
     dal::bit_vector nn;
     for (size_type i = 0; i < nbd; ++i) {
-      gmm::clear(e); e[i] = 1.0;
+      gmm::clear(e); e[i] = T(1);
       gmm::mult(D, e, aux);
-      scalar_type n = gmm::vect_norm2(aux);
+      R n = gmm::vect_norm2(aux);
 
-      if (n < 1.0E-8) { // scaler sur l'ensemble de D ...
-	G(i, nbase++) = 1.0; nn[i] = true;
+      if (n < 1e-8) { //norminfD*tol) {
+	G(i, nbase++) = T(1); nn[i] = true;
       }
       else {
 	bool good = true;
 	for (size_type j = 0; j < nb_bimg; ++j)
-	  if (dal::abs(gmm::vect_sp(aux, base_img[j])) > 1.0E-16)
+	  if (dal::abs(gmm::vect_sp(aux, base_img[j])) > T(0))
 	    { good = false; break; }
 	if (good) {
 	  gmm::copy(e, f);
-	  gmm::scale(f, 1.0 / n); gmm::scale(aux, 1.0 / n);
+	  gmm::scale(f, T(R(1)/n)); gmm::scale(aux, T(R(1)/n));
 	  base_img_inv[nb_bimg] = TEMP_VECT(nbd);
 	  gmm::copy(f, base_img_inv[nb_bimg]);
-	  gmm::clean(aux, 1.0E-18);
+	  gmm::clean(aux, gmm::vect_norminf(aux)*tol);
 	  base_img[nb_bimg] = TEMP_VECT(nbr);
 	  gmm::copy(aux, base_img[nb_bimg++]);
 	  nn[i] = true;
@@ -1131,23 +1053,23 @@ namespace getfem
 
     for (size_type i = 0; i < nbd; ++i)
       if (!(nn[i])) {
-	gmm::clear(e); e[i] = 1.0; gmm::clear(f); f[i] = 1.0;
+	gmm::clear(e); e[i] = T(1); gmm::clear(f); f[i] = T(1);
 	gmm::mult(D, e, aux);
 	for (size_type j = 0; j < nb_bimg; ++j) { 
-	  scalar_type c = gmm::vect_sp(aux, base_img[j]);
+	  T c = gmm::vect_sp(aux, base_img[j]);
 	  //	  if (dal::abs(c > 1.0E-6) { //  scaler sur l'ensemble de D ...
-	  if (c != 0.) {
+	  if (c != T(0)) {
 	    gmm::add(gmm::scaled(base_img[j], -c), aux);
 	    gmm::add(gmm::scaled(base_img_inv[j], -c), f);
 	  }
 	}
-	if (gmm::vect_norm2(aux) < 1.0E-8) { //  scaler sur l'ensemble de D ..
+	if (gmm::vect_norm2(aux) < norminfD*tol*R(10000)) {
 	  gmm::copy(f, gmm::mat_col(G, nbase++));
 	}
 	else {
-	  scalar_type n = gmm::vect_norm2(aux);
-	  gmm::scale(f, 1.0 / n); gmm::scale(aux, 1.0 / n);
-	  gmm::clean(f, 1.0E-18); gmm::clean(aux, 1.0E-18);
+	  R n = gmm::vect_norm2(aux);
+	  gmm::scale(f, T(R(1)/n)); gmm::scale(aux, T(R(1)/n));
+	  gmm::clean(f, tol*gmm::vect_norminf(f)); gmm::clean(aux, tol*gmm::vect_norminf(aux));
 	  base_img_inv[nb_bimg] = TEMP_VECT(nbd);
 	  gmm::copy(f, base_img_inv[nb_bimg]);
 	  base_img[nb_bimg] = TEMP_VECT(nbr);
@@ -1163,23 +1085,22 @@ namespace getfem
     // Orthogonalisation of the basis of the kernel of D.
     for (size_type i = nb_triv_base + 1; i < nbase; ++i) {
       for (size_type j = nb_triv_base; j < i; ++j) {
-	scalar_type c = gmm::vect_sp(gmm::mat_col(G,i), gmm::mat_col(G,j));
-	if (c != 0.)
+	T c = gmm::vect_sp(gmm::mat_col(G,i), gmm::mat_col(G,j));
+	if (c != T(0))
 	  gmm::add(gmm::scaled(gmm::mat_col(G,j), -c), gmm::mat_col(G,i));
       }
     }
     // projection of UDD on the orthogonal to the kernel.
     for (size_type j = nb_triv_base; j < nbase; ++j) {
-      scalar_type c = gmm::vect_sp(gmm::mat_col(G,j), UDD);
-      if (c != 0.)
+      T c = gmm::vect_sp(gmm::mat_col(G,j), UDD);
+      if (c != T(0))
 	gmm::add(gmm::scaled(gmm::mat_col(G,j), -c), UDD);
     }
     // Test ...
-    gmm::mult(D, UDD, gmm::scaled(UD, -1.0), aux);
-    if (gmm::vect_norm2(aux) > 1.0E-12)
+    gmm::mult(D, UDD, gmm::scaled(UD, T(-1)), aux);
+    if (gmm::vect_norm2(aux) > gmm::vect_norm2(UDD)*tol*R(10000))
       DAL_WARNING(2, "Dirichlet condition not well inverted: residu="
 		  << gmm::vect_norm2(aux));
-
     return nbase;
   }
   
@@ -1283,14 +1204,14 @@ namespace getfem
   /* ********************************************************************* */
   /*	Neumann Condition.                                                 */
   /* ********************************************************************* */
-
+#if 0
   template<class VECT1, class VECT2>
     void assembling_Neumann_condition(VECT1 &B, const mesh_fem &mf,
 				      size_type boundary, const mesh_fem &mfdata, const VECT2 &F, dim_type N)
   {
     DAL_WARNING(3, "obsolete function - use asm_source_term");
-    size_type cv, nbd1, nbd2, f;
-    dal::bit_vector nn = mf.convex_index(), nf;
+    size_type nbd1, nbd2, f;
+    dal::bit_vector nf;
     base_tensor t;
     pfem pf1, pf2, pf1prec = NULL, pf2prec = NULL;
     pintegration_method pim, pimprec = 0;
@@ -1301,8 +1222,7 @@ namespace getfem
       DAL_THROW(std::invalid_argument,
 		"This assembling procedure only works on a single mesh");
   
-    for (cv << nn; cv != ST_NIL; cv << nn)
-    {
+    for (dal::bv_visitor cv(mf.convex_index()); !cv.finished(); ++cv) {
       // for (int h = 0; h < mf.linked_mesh().nb_points_of_convex(cv); ++h)
       // cout << "Point " << h << " of cv " << cv << " : " 
       //     << mf.linked_mesh().points_of_convex(cv)[h] << endl;
@@ -1329,7 +1249,8 @@ namespace getfem
 	{
 	  // cout << "cv = " << cv << " f = " << f << endl;
 
-	  pmec->gen_compute_on_face(t,mf.linked_mesh().points_of_convex(cv),f);
+	  pmec->gen_compute_on_face(t,mf.linked_mesh().points_of_convex(cv), 
+				    f, cv);
 	  // cout << "t = " << t << endl;
 	  base_tensor::iterator p = t.begin();
 	  for (size_type i = 0; i < nbd2; i++)
@@ -1348,9 +1269,9 @@ namespace getfem
       }
     }
   }
-
+#endif
   
 }  /* end of namespace getfem.                                             */
 
 
-#endif /* __GETFEM_ASSEMBLING_H  */
+#endif /* GETFEM_ASSEMBLING_H__  */
