@@ -30,8 +30,8 @@
 /* *********************************************************************** */
 
 
-#ifndef __GETFEM_FEM_H
-#define __GETFEM_FEM_H
+#ifndef GETFEM_FEM_H__
+#define GETFEM_FEM_H__
 
 #include <bgeot_geometric_trans.h>
 #include <getfem_integration.h>
@@ -78,7 +78,8 @@ namespace getfem
   bool dof_linkable(pdof_description);
   /// Says if the two dofs can be identified.
   bool dof_compatibility(pdof_description, pdof_description);
-
+  /// Returns the xfem_index of dof (0 for normal dof)
+  size_type dof_xfem_index(pdof_description);
   
   /* ******************************************************************** */
   /*	Classes for description of a finite element.                      */
@@ -91,7 +92,7 @@ namespace getfem
   {
   protected :
 
-    std::vector<pdof_description> _dof_types;
+    std::vector<pdof_description> dof_types_;
     bgeot::convex_structure cvs_node;
     bgeot::convex<base_node> cv_node;
     mutable bgeot::pstored_point_tab pspt;
@@ -103,7 +104,7 @@ namespace getfem
     
   public :
     /// Number of degrees of freedom.
-    virtual size_type nb_dof(void) const { return _dof_types.size(); }
+    virtual size_type nb_dof(void) const { return dof_types_.size(); }
     virtual size_type nb_base(void) const { return nb_dof(); }
     /// Number of components (nb_dof() * dimension of the target space).
     size_type nb_base_components(void) const
@@ -112,7 +113,7 @@ namespace getfem
       { return nb_dof() * ntarget_dim; }
     /// Gives the array of pointer on dof description.
     const std::vector<pdof_description> &dof_types(void) const 
-      { return _dof_types; }
+      { return dof_types_; }
     short_type hierarchical_raff(void) const { return hier_raff; }
     /// dimension of the reference element.
     dim_type dim(void) const { return cvr->structure()->dim(); }
@@ -153,6 +154,7 @@ namespace getfem
     bool &is_polynomial(void) { return is_pol; }
     short_type estimated_degree(void) const { return es_degree; }
     short_type &estimated_degree(void) { return es_degree; }
+    
     virtual void mat_trans(base_matrix &, const base_matrix &,
 			   bgeot::pgeometric_trans) const
       { DAL_THROW(internal_error, "This function should not be called."); }
@@ -165,7 +167,7 @@ namespace getfem
     virtual void interpolation(const base_node &x, const base_matrix &G,
 			       bgeot::pgeometric_trans pgt,
 			       const base_vector &coeff, 
-			       base_node &val) const = 0;
+			       base_vector &val) const = 0;
     /** Function which interpolates in the ii th point of pfp. coeff is the
      *  vector of coefficient relatively to the shape functions. G and pgt
      *  represent the geometric transformation for non-equivalent elements.
@@ -176,12 +178,7 @@ namespace getfem
 			       const base_matrix &G,
 			       bgeot::pgeometric_trans pgt, 
 			       const base_vector &coeff, 
-			       base_node &val, dim_type Qdim=1) const;
-    //     virtual void interpolation_grad(const base_node &x, 
-    //                                     const base_matrix &G,
-    // 				    bgeot::pgeometric_trans pgt,
-    // 				    const base_vector &coeff,
-    // 				    base_matrix &val) const;
+			       base_vector &val, dim_type Qdim=1) const;
     /** Function which interpolates the gradient in a arbitrary point x
      *  given on the reference element. coeff is the vector of coefficient
      *  relatively to the shape functions. G and pgt
@@ -225,7 +222,7 @@ namespace getfem
      */
     virtual void real_base_value(pgeotrans_precomp pgp, pfem_precomp pfp,
 				 size_type ii, const base_matrix &G,
-				 base_tensor &t) const;
+				 base_tensor &t, size_type elt) const;
     /** Gives the value of all gradients on the real element of the components
      *  of the base functions at the point ii of the pgp possibly using
      *  information on pfp, G and B. B is the matrix wich transforms the
@@ -236,7 +233,7 @@ namespace getfem
     virtual void real_grad_base_value(pgeotrans_precomp pgp, pfem_precomp pfp,
 				      size_type ii, const base_matrix &G,
 				      const base_matrix &B,
-				      base_tensor &t) const;
+				      base_tensor &t, size_type elt) const;
     /** Gives the value of all hessians on the real element of the components
      *  of the base functions at the point ii of the pgp possibly using
      *  information on pfp, G, B2 and B32. B2 and B32 are the matrices used
@@ -248,7 +245,7 @@ namespace getfem
 				      size_type ii, const base_matrix &G,
 				      const base_matrix &B3,
 				      const base_matrix &B32,
-				      base_tensor &t) const;
+				      base_tensor &t, size_type elt) const;
 
     
     virtual size_type index_of_already_numerate_dof(size_type, size_type) const
@@ -265,7 +262,7 @@ namespace getfem
     void unfreeze_cvs_node(void);
 
     virtual_fem &operator =(const virtual_fem &f) {
-      _dof_types = f._dof_types;
+      dof_types_ = f.dof_types_;
       cvs_node = f.cvs_node;
       cv_node = f.cv_node;
       cv_node.structure() = &(cvs_node);
@@ -292,25 +289,25 @@ namespace getfem
   
   template <class FUNC> class fem : public virtual_fem {
   protected :
-    std::vector<FUNC> _base;
+    std::vector<FUNC> base_;
     
   public :
     
     /// Gives the array of basic functions (components).
-    const std::vector<FUNC> &base(void) const { return _base; }
-    std::vector<FUNC> &base(void) { return _base; }
+    const std::vector<FUNC> &base(void) const { return base_; }
+    std::vector<FUNC> &base(void) { return base_; }
     
     /* just to please for HP aCC and SGI CC */
     virtual void interpolation(pfem_precomp pfp, size_type ii,
 			       const base_matrix &G,
 			       bgeot::pgeometric_trans pgt, 
 			       const base_vector &coeff, 
-			       base_node &val, dim_type Qdim=1) const { 
+			       base_vector &val, dim_type Qdim=1) const { 
       virtual_fem::interpolation(pfp,ii,G,pgt,coeff,val,Qdim); 
     }
     void interpolation(const base_node &x, const base_matrix &G, 
 		       bgeot::pgeometric_trans pgt,
-		       const base_vector &coeff, base_node &val) const;
+		       const base_vector &coeff, base_vector &val) const;
 
     void base_value(const base_node &x, base_tensor &t) const {
       bgeot::multi_index mi(2);
@@ -319,7 +316,7 @@ namespace getfem
       size_type R = nb_base_components();
       base_tensor::iterator it = t.begin();
       for (size_type  i = 0; i < R; ++i, ++it)
-	*it = _base[i].eval(x.begin());
+	*it = base_[i].eval(x.begin());
     }
     void grad_base_value(const base_node &x, base_tensor &t) const {
       bgeot::multi_index mi(3);
@@ -330,7 +327,7 @@ namespace getfem
       base_tensor::iterator it = t.begin();
       for (dim_type j = 0; j < n; ++j)
 	for (size_type i = 0; i < R; ++i, ++it)
-	  { FUNC f = _base[i]; f.derivative(j); *it = f.eval(x.begin()); }
+	  { FUNC f = base_[i]; f.derivative(j); *it = f.eval(x.begin()); }
     }
     void hess_base_value(const base_node &x, base_tensor &t) const {
       bgeot::multi_index mi(4);
@@ -342,7 +339,7 @@ namespace getfem
       for (dim_type k = 0; k < n; ++k)
 	for (dim_type j = 0; j < n; ++j)
 	  for (size_type i = 0; i < R; ++i, ++it) {
-	    FUNC f = _base[i];
+	    FUNC f = base_[i];
 	    f.derivative(j); f.derivative(k);
 	    *it = f.eval(x.begin());
 	  }
@@ -358,7 +355,7 @@ namespace getfem
   void fem<FUNC>::interpolation(const base_node &x, const base_matrix &G,
 				bgeot::pgeometric_trans pgt, 
 				const base_vector &coeff,
-				base_node &val) const { 
+				base_vector &val) const { 
     // optimisable.   verifier et faire le vectoriel
     base_matrix M;
     if (val.size() != target_dim())
