@@ -31,8 +31,8 @@
 /* *********************************************************************** */
 
 
-#ifndef __GETFEM_MESH_H
-#define __GETFEM_MESH_H
+#ifndef GETFEM_MESH_H__
+#define GETFEM_MESH_H__
 
 #include <bgeot_mesh.h>
 #include <bgeot_geometric_trans.h>
@@ -212,6 +212,7 @@ namespace getfem
       typedef lmsg::linkmsg_sender<getfem_mesh_receiver> msg_sender;
 
     protected :
+    /* if a new field is added here, do NOT forget to add it in the copy_from method! */
 
       double eps_p;  /* infinity distance under wich two points are equal. */
       msg_sender lkmsg; /* gestionnaire de msg.                            */
@@ -350,12 +351,15 @@ namespace getfem
 
       /* returns the normal of face 'f' evaluated at the point 'pt'       */
       /* (pt is a position in the reference convex)                       */
-      base_vector normal_of_face_of_convex(size_type ic, short_type f,
-					   const base_node &pt) const;
-      base_vector normal_of_face_of_convex(size_type ic, short_type f,
-					   size_type n) const;
+      base_small_vector normal_of_face_of_convex(size_type ic, short_type f,
+						 const base_node &pt) const;
+      base_small_vector normal_of_face_of_convex(size_type ic, short_type f,
+						 size_type n) const;
 
-      void translation(base_vector);
+      scalar_type convex_quality_estimate(size_type ic) const;
+      scalar_type convex_radius_estimate(size_type ic) const;
+      scalar_type minimal_convex_radius_estimate() const;
+      void translation(base_small_vector);
       void transformation(base_matrix);
   
       void optimize_structure(void);
@@ -365,15 +369,9 @@ namespace getfem
       void write_to_file(std::ostream &ost) const;
       void read_from_file(const std::string &name);
       void read_from_file(std::istream &ist);
-    
-    size_type memsize() const {
-      return bgeot::mesh<base_node>::memsize() + 
-	(pts.index().last_true()+1)*dim()*sizeof(scalar_type)+
-	sizeof(getfem_mesh) - sizeof(bgeot::mesh<base_node>)
-	+trans_exists.memsize() + gtab.memsize();
-    }
-    
-    ~getfem_mesh() { lmsg_sender().send(MESH_DELETE()); }
+      void copy_from(const getfem_mesh& m); /* might be the copy constructor */
+      size_type memsize() const;
+      ~getfem_mesh() { lmsg_sender().send(MESH_DELETE()); }
   };
 
   template<class ITER>
@@ -434,7 +432,15 @@ namespace getfem
 
   typedef getfem_mesh *pgetfem_mesh;
 
+  /** rough estimate of the maximum value of the condition 
+   * number of the jacobian of the geometric transformation */
+  scalar_type convex_quality_estimate(bgeot::pgeometric_trans pgt, const base_matrix& pts);
+
+  /** rough estimate of the radius of the convex using the largest eigenvalue
+   * of the jacobian of the geometric transformation */
+  scalar_type convex_radius_estimate(bgeot::pgeometric_trans pgt, const base_matrix& pts);
+  
 }  /* end of namespace getfem.                                             */
 
 
-#endif /* __GETFEM_MESH_H  */
+#endif /* GETFEM_MESH_H__  */
