@@ -3,10 +3,8 @@
 #define GETFEM_MESH_SLICES_H
 
 #include <bitset>
-#include <gmm.h>
-#include <getfem_mesh.h>
-#include <getfem_fem.h>
-#include <getfem_poly_composite.h>
+#include <gmm_kernel.h>
+#include <getfem_mesh_fem.h>
 #include <memory> // auto_ptr for g++ 2.95
 
 namespace getfem {
@@ -79,7 +77,7 @@ namespace getfem {
     std::vector<size_type> simplex_cnt; // count simplexes of dimension 0,1,...,dim
     size_type points_cnt;
     cvlst_ct cvlst;
-    size_type _dim;
+    size_type dim_;
     void do_slicing(size_type cv, bgeot::pconvex_ref cvr, slicer *ms, cs_nodes_ct cv_nodes, 
 		    cs_simplexes_ct cv_simplexes, dal::bit_vector& splx_in);
   protected:
@@ -125,7 +123,7 @@ namespace getfem {
     size_type nb_convex() const { return cvlst.size(); }
     size_type convex_num(size_type ic) const { return cvlst[ic].cv_num; }
     void set_dim(size_type newdim);
-    size_type dim() const { return _dim; }
+    size_type dim() const { return dim_; }
     const getfem_mesh& linked_mesh() const { return m; }
     void nb_simplexes(std::vector<size_type>& c) const { c = simplex_cnt; }
     size_type nb_simplexes(size_type sdim) const { return simplex_cnt[sdim]; }
@@ -139,23 +137,27 @@ namespace getfem {
     /** merges with another mesh slice */
     void merge(const mesh_slice& sl);
 
+    /** builds a P1 mesh from the slice */
+    void to_mesh(getfem_mesh& m) const; 
+
     /** interpolation of a mesh_fem on a slice (the mesh_fem
 	and the slice must share the same mesh, of course)
     */
     template<typename V1, typename V2> void 
     interpolate(const getfem::mesh_fem &mf, const V1& U, V2& V) {
-      _fem_precomp fprecomp;
+      fem_precomp_ fprecomp;
       bgeot::stored_point_tab refpts;
       base_vector coeff;
       base_matrix G;
-      base_node val(mf.get_qdim());
+      base_vector val(mf.get_qdim());
       typename V2::iterator out = V.begin();
       for (size_type i=0; i < nb_convex(); ++i) {
         size_type cv = convex_num(i);
         refpts.resize(nodes(i).size());
         for (size_type j=0; j < refpts.size(); ++j) refpts[j] = nodes(i)[j].pt_ref;
         pfem pf = mf.fem_of_element(cv);
-	if (pf->need_G())transfert_to_G(G, mf.linked_mesh().points_of_convex(cv));
+	if (pf->need_G()) 
+	  bgeot::vectors_to_base_matrix(G, mf.linked_mesh().points_of_convex(cv));
         fem_precomp_not_stored(pf, &refpts, fprecomp);
         
         ref_mesh_dof_ind_ct dof = mf.ind_dof_of_element(cv);
@@ -335,6 +337,9 @@ namespace getfem {
       return slicer_volume::trinom(a,b,c);
     }
   public:
+    /* orient = -1 => select interior, 
+       orient = 0  => select boundary 
+       orient = +1 => select exterior */
     slicer_sphere(base_node x0_, scalar_type R_, int orient_) : 
       slicer_volume(orient_), x0(x0_), R(R_) {} //cerr << "slicer_volume, x0=" << x0 << ", R=" << R << endl; }
   };
@@ -455,7 +460,7 @@ namespace getfem {
 	       mesh_slice::cs_simplexes_ct& /*splxs*/, dal::bit_vector& /*splx_in*/) {}
   };
   
-
+  std::ostream& operator<<(std::ostream& o, const mesh_slice& m);
 }
 
 #endif /*GETFEM_MESH_SLICES_H*/
