@@ -13,7 +13,7 @@
 /*                                                                         */
 /* Copyright (C) 2002  Yves Renard.                                        */
 /*                                                                         */
-/* This file is a part of GETFEM++                                         */
+/* This file is a part of GMM++                                            */
 /*                                                                         */
 /* This program is free software; you can redistribute it and/or modify    */
 /* it under the terms of the GNU Lesser General Public License as          */
@@ -53,8 +53,11 @@
 // OR OTHER RIGHTS.
 //=======================================================================
 
-#ifndef __GMM_SOLVER_BICGSTAB_H
-#define __GMM_SOLVER_BICGSTAB_H
+#ifndef GMM_SOLVER_BICGSTAB_H__
+#define GMM_SOLVER_BICGSTAB_H__
+
+#include <gmm_kernel.h>
+#include <gmm_precond_diagonal.h>
 
 namespace gmm {
 
@@ -63,11 +66,13 @@ namespace gmm {
   /* (preconditionned, with parametrable scalar product)        	  */
   /* ******************************************************************** */
 
-  template <class Matrix, class Vector, class VectorB, class Preconditioner>
+  template <typename Matrix, typename Vector, typename VectorB,
+	    typename Preconditioner>
   void bicgstab(const Matrix& A, Vector& x, const VectorB& b,
 	       const Preconditioner& M, iteration &iter) {
 
     typedef typename linalg_traits<Vector>::value_type T;
+    typedef typename number_traits<T>::magnitude_type R;
     typedef typename temporary_dense_vector<Vector>::vector_type temp_vector;
     
     T rho_1, rho_2(0), alpha(0), beta, omega(0);
@@ -75,62 +80,65 @@ namespace gmm {
       shat(vect_size(x)), 
       t(vect_size(x)), v(vect_size(x)), r(vect_size(x)), rtilde(vect_size(x));
     
-    gmm::mult(A, gmm::scaled(x, -1.0), b, r);	  
+    gmm::mult(A, gmm::scaled(x, -T(1)), b, r);	  
     gmm::copy(r, rtilde);
-    T norm_r = gmm::vect_norm2(r);
+    R norm_r = gmm::vect_norm2(r);
     iter.set_rhsnorm(gmm::vect_norm2(b));
 
-    if (iter.get_rhsnorm() == 0.0)
-      clear(x);
-    else {
-      while (!iter.finished(norm_r)) {
-	
-	rho_1 = gmm::vect_sp(rtilde, r);
-	if (rho_1 == T(0))
-	  DAL_THROW(failure_error, "Bicgstab failed to converge");
-	
-	if (iter.first())
-	  gmm::copy(r, p);
-	else {
-	  if (omega == T(0))
-	    DAL_THROW(failure_error, "Bicgstab failed to converge");
-	  
-	  beta = (rho_1 / rho_2) * (alpha / omega);
-	  
-	  gmm::add(gmm::scaled(v, -omega), p); // c'est bon a ? 
-	  gmm::add(r, gmm::scaled(p, beta), p);      
-	}
-	gmm::mult(M, p, phat);
-	gmm::mult(A, phat, v);	
-	alpha = rho_1 / gmm::vect_sp(v, rtilde);
-	gmm::add(r, gmm::scaled(v, -alpha), s);
-	
-	if (iter.finished_vect(s)) {
-	  gmm::add(gmm::scaled(phat, alpha), x); 
-	  break;
-	}
-	
-	gmm::mult(M, s, shat);	
-	gmm::mult(A, shat, t);	
-	omega = gmm::vect_sp(t, s) / gmm::vect_sp(t, t);
-	
-	gmm::add(gmm::scaled(phat, alpha), x); 
-	gmm::add(gmm::scaled(shat, omega), x);
-	gmm::add(s, gmm::scaled(t, -omega), r); 
-	norm_r = gmm::vect_norm2(r);
-	rho_2 = rho_1;
-	
-	++iter;
+    if (iter.get_rhsnorm() == 0.0) { clear(x); return; }
+    
+    while (!iter.finished(norm_r)) {
+      
+      rho_1 = gmm::vect_sp(rtilde, r);
+      if (rho_1 == T(0)) {
+	if (iter.get_maxiter() == size_type(-1)) 
+	  { DAL_THROW(failure_error, "Bicgstab failed to converge"); }
+	else { DAL_WARNING(1, "Bicgstab failed to converge"); return; }
       }
+      
+      if (iter.first())
+	gmm::copy(r, p);
+      else {
+	if (omega == T(0)) {
+	  if (iter.get_maxiter() == size_type(-1))
+	    { DAL_THROW(failure_error, "Bicgstab failed to converge"); }
+	  else { DAL_WARNING(1, "Bicgstab failed to converge"); return; }
+	}
+	
+	beta = (rho_1 / rho_2) * (alpha / omega);
+	
+	gmm::add(gmm::scaled(v, -omega), p);
+	gmm::add(r, gmm::scaled(p, beta), p);      
+      }
+      gmm::mult(M, p, phat);
+      gmm::mult(A, phat, v);	
+      alpha = rho_1 / gmm::vect_sp(v, rtilde);
+      gmm::add(r, gmm::scaled(v, -alpha), s);
+      
+      if (iter.finished_vect(s)) 
+	{ gmm::add(gmm::scaled(phat, alpha), x); break; }
+      
+      gmm::mult(M, s, shat);	
+      gmm::mult(A, shat, t);
+      omega = gmm::vect_sp(t, s) / gmm::vect_norm2_sqr(t);
+      
+      gmm::add(gmm::scaled(phat, alpha), x); 
+      gmm::add(gmm::scaled(shat, omega), x);
+      gmm::add(s, gmm::scaled(t, -omega), r); 
+      norm_r = gmm::vect_norm2(r);
+      rho_2 = rho_1;
+      
+      ++iter;
     }
   }
-
-  template <class Matrix, class Vector, class VectorB, class Preconditioner>
-  int bicgstab(const Matrix& A, const Vector& x, const VectorB& b,
+  
+  template <typename Matrix, typename Vector, typename VectorB,
+	    typename Preconditioner>
+  void bicgstab(const Matrix& A, const Vector& x, const VectorB& b,
 	       const Preconditioner& M, iteration &iter)
   { bicgstab(A, linalg_const_cast(x), b, M, iter); }
   
 }
 
 
-#endif //  __GMM_SOLVER_BICGSTAB_H
+#endif //  GMM_SOLVER_BICGSTAB_H__
