@@ -12,7 +12,7 @@
 /*                                                                         */
 /* Copyright (C) 2002  Yves Renard.                                        */
 /*                                                                         */
-/* This file is a part of GETFEM++                                         */
+/* This file is a part of GMM++                                            */
 /*                                                                         */
 /* This program is free software; you can redistribute it and/or modify    */
 /* it under the terms of the GNU Lesser General Public License as          */
@@ -57,12 +57,17 @@
 
 //  preconditionning does not work
 
-#ifndef __GMM_SOLVER_CCG_H
-#define __GMM_SOLVER_CCG_H
+#ifndef GMM_SOLVER_CCG_H__
+#define GMM_SOLVER_CCG_H__
+
+#include <gmm_kernel.h>
+#include <gmm_precond_diagonal.h>
+
 
 namespace gmm {
 
-  template <class CMatrix, class CINVMatrix, class Matps, class VectorX>
+  template <typename CMatrix, typename CINVMatrix, typename Matps,
+	    typename VectorX>
   void pseudo_inverse(const CMatrix &C, CINVMatrix &CINV,
 		      const Matps& /* PS */, VectorX&) {
     // compute the pseudo inverse of the non-square matrix C such
@@ -111,8 +116,9 @@ namespace gmm {
   
   // Compute the minimum of 1/2((Ax).x) - bx under the contraint Cx <= f
 
-  template < class Matrix, class CMatrix, class Matps, class VectorX,
-	     class VectorB, class VectorF, class Preconditioner >
+  template < typename Matrix,  typename CMatrix, typename Matps,
+	     typename VectorX, typename VectorB, typename VectorF,
+	     typename Preconditioner >
   void constrained_cg(const Matrix& A, const CMatrix& C, VectorX& x,
 		      const VectorB& b, const VectorF& f,const Matps& PS,
 		      const Preconditioner& M, iteration &iter) {
@@ -182,4 +188,4 @@ namespace gmm {
   
 }
 
-#endif //  __GMM_SOLVER_CCG_H
+#endif //  GMM_SOLVER_CCG_H__
