@@ -11,7 +11,7 @@
 /*                                                                         */
 /* Copyright (C) 2002  Yves Renard.                                        */
 /*                                                                         */
-/* This file is a part of GETFEM++                                         */
+/* This file is a part of GMM++                                            */
 /*                                                                         */
 /* This program is free software; you can redistribute it and/or modify    */
 /* it under the terms of the GNU Lesser General Public License as          */
@@ -29,8 +29,10 @@
 /*                                                                         */
 /* *********************************************************************** */
 
-#ifndef __GMM_SUB_MATRIX_H
-#define __GMM_SUB_MATRIX_H
+#ifndef GMM_SUB_MATRIX_H__
+#define GMM_SUB_MATRIX_H__
+
+#include <gmm_sub_vector.h>
 
 namespace gmm {
 
@@ -38,66 +40,48 @@ namespace gmm {
   /*		sub row matrices type                                      */
   /* ********************************************************************* */
 
-  template <class PT, class SUBI1, class SUBI2> struct gen_sub_row_matrix {
+  template <typename PT, typename SUBI1, typename SUBI2>
+  struct gen_sub_row_matrix {
     typedef gen_sub_row_matrix<PT, SUBI1, SUBI2> this_type;
     typedef typename std::iterator_traits<PT>::value_type M;
     typedef M * CPT;
     typedef typename std::iterator_traits<PT>::reference ref_M;
-    typedef typename select_return<typename linalg_traits<M>
+    typedef typename select_ref<typename linalg_traits<M>
             ::const_row_iterator, typename linalg_traits<M>::row_iterator,
-	    PT>::return_type iterator;
+	    PT>::ref_type iterator;
     typedef typename linalg_traits<this_type>::reference reference;
-    typedef typename linalg_traits<M>::access_type access_type;
+    typedef typename linalg_traits<this_type>::porigin_type porigin_type;
 
     SUBI1 si1;
     SUBI2 si2;
-    iterator _begin;
-    const void *origin;
+    iterator begin_;
+    porigin_type origin;
     
-    reference operator()(size_type i, size_type j) const
-    { return access_type()(_begin + si1.index(i), si2.index(j)); }
+    reference operator()(size_type i, size_type j) const 
+    { return linalg_traits<M>::access(begin_ + si1.index(i), si2.index(j)); }
    
     size_type nrows(void) const { return si1.size(); }
     size_type ncols(void) const { return si2.size(); }
     
     gen_sub_row_matrix(ref_M m, const SUBI1 &s1, const SUBI2 &s2)
-      : si1(s1), si2(s2), _begin(mat_row_begin(m)),
+      : si1(s1), si2(s2), begin_(mat_row_begin(m)),
 	origin(linalg_origin(m)) {}
     gen_sub_row_matrix() {}
     gen_sub_row_matrix(const gen_sub_row_matrix<CPT, SUBI1, SUBI2> &cr) :
-      si1(cr.si1), si2(cr.si2), _begin(cr._begin),origin(cr.origin) {}
+      si1(cr.si1), si2(cr.si2), begin_(cr.begin_),origin(cr.origin) {}
   };
 
-  template <class PT, class SUBI1, class SUBI2> 
-  struct gen_sub_row_matrix_access {
-    typedef gen_sub_row_matrix<PT, SUBI1, SUBI2> this_type;
-    typedef typename std::iterator_traits<PT>::value_type M;
-    typedef typename linalg_traits<this_type>::reference reference;
-    typedef typename linalg_traits<this_type>::value_type value_type;
-    typedef typename linalg_traits<M>::access_type access_type;
-    typedef typename linalg_traits<this_type>::const_row_iterator const_iter;
-    typedef typename linalg_traits<this_type>::row_iterator iterator;
-    
-    reference operator()(const iterator &itrow, size_type i)
-    { return access_type()(*itrow, itrow.si2.index(i)); }
-
-    value_type operator()(const const_iter &itrow, size_type i)
-    { return access_type()(*itrow, itrow.si2.index(i)); }
-    
-  };
-
-  template <class PT, class SUBI1, class SUBI2>
+  template <typename PT, typename SUBI1, typename SUBI2>
   struct gen_sub_row_matrix_iterator {
     typedef gen_sub_row_matrix<PT, SUBI1, SUBI2> this_type;
     typedef typename modifiable_pointer<PT>::pointer MPT;
     typedef typename std::iterator_traits<PT>::value_type M;
-    typedef typename select_return<typename linalg_traits<M>
+    typedef typename select_ref<typename linalg_traits<M>
             ::const_row_iterator, typename linalg_traits<M>::row_iterator,
-	    PT>::return_type ITER;
+	    PT>::ref_type ITER;
     typedef ITER value_type;
     typedef ITER *pointer;
     typedef ITER &reference;
-    typedef typename linalg_traits<M>::access_type access_type;
     typedef ptrdiff_t difference_type;
     typedef size_t size_type;
     typedef std::random_access_iterator_tag  iterator_category;
@@ -137,15 +121,18 @@ namespace gmm {
     
   };
 
-  template <class PT, class SUBI1, class SUBI2>
+  template <typename PT, typename SUBI1, typename SUBI2>
   struct linalg_traits<gen_sub_row_matrix<PT, SUBI1, SUBI2> > {
     typedef gen_sub_row_matrix<PT, SUBI1, SUBI2> this_type;
     typedef typename std::iterator_traits<PT>::value_type M;
     typedef typename which_reference<PT>::is_reference is_reference;
     typedef abstract_matrix linalg_type;
+    typedef typename linalg_traits<M>::origin_type origin_type;
+    typedef typename select_ref<const origin_type *, origin_type *,
+				PT>::ref_type porigin_type;
     typedef typename linalg_traits<M>::value_type value_type;
-    typedef typename select_return<value_type,
-            typename linalg_traits<M>::reference, PT>::return_type reference;
+    typedef typename select_ref<value_type,
+            typename linalg_traits<M>::reference, PT>::ref_type reference;
     typedef abstract_null_type sub_col_type;
     typedef abstract_null_type col_iterator;
     typedef abstract_null_type const_sub_col_type;
@@ -153,16 +140,16 @@ namespace gmm {
     typedef typename sub_vector_type<const typename
             linalg_traits<M>::const_sub_row_type *, SUBI2>::vector_type
             const_sub_row_type;
-    typedef typename select_return<abstract_null_type, 
+    typedef typename select_ref<abstract_null_type, 
             typename sub_vector_type<typename linalg_traits<M>::sub_row_type *,
-	    SUBI2>::vector_type, PT>::return_type sub_row_type;
+	    SUBI2>::vector_type, PT>::ref_type sub_row_type;
     typedef gen_sub_row_matrix_iterator<typename const_pointer<PT>::pointer,
 	    SUBI1, SUBI2> const_row_iterator;
-    typedef typename select_return<abstract_null_type, 
-	    gen_sub_row_matrix_iterator<PT, SUBI1, SUBI2>, PT>::return_type
+    typedef typename select_ref<abstract_null_type, 
+	    gen_sub_row_matrix_iterator<PT, SUBI1, SUBI2>, PT>::ref_type
             row_iterator;
-    typedef typename linalg_traits<const_sub_row_type>::storage_type storage_type;
-    typedef gen_sub_row_matrix_access<PT, SUBI1, SUBI2> access_type;
+    typedef typename linalg_traits<const_sub_row_type>::storage_type
+            storage_type;
     typedef row_major sub_orientation;
     static size_type nrows(const this_type &m) { return m.nrows(); }
     static size_type ncols(const this_type &m) { return m.ncols(); }
@@ -171,94 +158,77 @@ namespace gmm {
     static sub_row_type row(const row_iterator &it)
     { return sub_row_type(linalg_traits<M>::row(*it), it.si2); }
     static const_row_iterator row_begin(const this_type &m)
-    { return const_row_iterator(m._begin, m.si1, m.si2, 0); }
+    { return const_row_iterator(m.begin_, m.si1, m.si2, 0); }
     static row_iterator row_begin(this_type &m)
-    { return row_iterator(m._begin, m.si1, m.si2, 0); }
+    { return row_iterator(m.begin_, m.si1, m.si2, 0); }
     static const_row_iterator row_end(const this_type &m)
-    { return const_row_iterator(m._begin, m.si1, m.si2,  m.nrows()); }
+    { return const_row_iterator(m.begin_, m.si1, m.si2,  m.nrows()); }
     static row_iterator row_end(this_type &m)
-    { return row_iterator(m._begin, m.si1, m.si2, m.nrows()); }    
-    static const void* origin(const this_type &m) { return m.origin; }
+    { return row_iterator(m.begin_, m.si1, m.si2, m.nrows()); }
+    static origin_type* origin(this_type &v) { return v.origin; }
+    static const origin_type* origin(const this_type &v) { return v.origin; }
     static void do_clear(this_type &m) {
       row_iterator it = mat_row_begin(m), ite = mat_row_end(m);
       for (; it != ite; ++it) clear(row(it));
     }
+    static value_type access(const const_row_iterator &itrow, size_type i)
+    { return linalg_traits<M>::access(*itrow, itrow.si2.index(i)); }
+    static reference access(const row_iterator &itrow, size_type i)
+    { return linalg_traits<M>::access(*itrow, itrow.si2.index(i)); }
   };
   
-  template <class PT, class SUBI1, class SUBI2> std::ostream &operator <<
-  (std::ostream &o, const gen_sub_row_matrix<PT, SUBI1, SUBI2>& m)
+  template <typename PT, typename SUBI1, typename SUBI2>
+  std::ostream &operator <<(std::ostream &o,
+			    const gen_sub_row_matrix<PT, SUBI1, SUBI2>& m)
   { gmm::write(o,m); return o; }
 
-#ifdef USING_BROKEN_GCC295
-  template <class PT, class SUBI1, class SUBI2>
-  struct linalg_traits<const gen_sub_row_matrix<PT, SUBI1, SUBI2> >
-    : public linalg_traits<gen_sub_row_matrix<PT, SUBI1, SUBI2> > {};
-#endif
 
   /* ********************************************************************* */
   /*		sub column matrices type                                   */
   /* ********************************************************************* */
 
-  template <class PT, class SUBI1, class SUBI2> struct gen_sub_col_matrix {
+  template <typename PT, typename SUBI1, typename SUBI2>
+  struct gen_sub_col_matrix {
     typedef gen_sub_col_matrix<PT, SUBI1, SUBI2> this_type;
     typedef typename std::iterator_traits<PT>::value_type M;
     typedef M * CPT;
     typedef typename std::iterator_traits<PT>::reference ref_M;
-    typedef typename select_return<typename linalg_traits<M>
+    typedef typename select_ref<typename linalg_traits<M>
             ::const_col_iterator, typename linalg_traits<M>::col_iterator,
-	    PT>::return_type iterator;
+	    PT>::ref_type iterator;
     typedef typename linalg_traits<this_type>::reference reference;
-    typedef typename linalg_traits<M>::access_type access_type;
+    typedef typename linalg_traits<this_type>::porigin_type porigin_type;
 
     SUBI1 si1;
     SUBI2 si2;
-    iterator _begin;
-    const void *origin;
+    iterator begin_;
+    porigin_type origin;
     
     reference operator()(size_type i, size_type j) const
-    { // cout << "to(" << si1.index(i) << "," << si2.index(j) << ")";
-    return access_type()(_begin + si2.index(j), si1.index(i)); }
+    { return linalg_traits<M>::access(begin_ + si2.index(j), si1.index(i)); }
 
     size_type nrows(void) const { return si1.size(); }
     size_type ncols(void) const { return si2.size(); }
     
     gen_sub_col_matrix(ref_M m, const SUBI1 &s1, const SUBI2 &s2)
-      : si1(s1), si2(s2), _begin(mat_col_begin(m)),
+      : si1(s1), si2(s2), begin_(mat_col_begin(m)),
         origin(linalg_origin(m)) {}
     gen_sub_col_matrix() {}
     gen_sub_col_matrix(const gen_sub_col_matrix<CPT, SUBI1, SUBI2> &cr) :
-      si1(cr.si1), si2(cr.si2), _begin(cr._begin),origin(cr.origin) {}
+      si1(cr.si1), si2(cr.si2), begin_(cr.begin_),origin(cr.origin) {}
   };
 
-  template <class PT, class SUBI1, class SUBI2> 
-  struct gen_sub_col_matrix_access {
-    typedef gen_sub_col_matrix<PT, SUBI1, SUBI2> this_type;
-    typedef typename std::iterator_traits<PT>::value_type M;
-    typedef typename linalg_traits<this_type>::reference reference;
-    typedef typename linalg_traits<this_type>::value_type value_type;
-    typedef typename linalg_traits<M>::access_type access_type;
-    typedef typename linalg_traits<this_type>::const_col_iterator const_iter;
-    typedef typename linalg_traits<this_type>::col_iterator iterator;
-    
-    reference operator()(const iterator &itcol, size_type i)
-    { return access_type()(*itcol, itcol.si2.index(i)); }
-
-    value_type operator()(const const_iter &itcol, size_type i)
-    { return access_type()(*itcol, itcol.si2.index(i)); }
-  };
-
-  template <class PT, class SUBI1, class SUBI2>
+  template <typename PT, typename SUBI1, typename SUBI2>
   struct gen_sub_col_matrix_iterator {
     typedef gen_sub_col_matrix<PT, SUBI1, SUBI2> this_type;
     typedef typename modifiable_pointer<PT>::pointer MPT;
     typedef typename std::iterator_traits<PT>::value_type M;
-    typedef typename select_return<typename linalg_traits<M>
-            ::const_col_iterator, typename linalg_traits<M>::col_iterator,
-	    PT>::return_type ITER;
+    typedef typename select_ref<typename linalg_traits<M>::const_col_iterator,
+				typename linalg_traits<M>::col_iterator,
+				PT>::ref_type ITER;
     typedef ITER value_type;
     typedef ITER *pointer;
     typedef ITER &reference;
-    typedef typename linalg_traits<M>::access_type access_type;
     typedef ptrdiff_t difference_type;
     typedef size_t size_type;
     typedef std::random_access_iterator_tag  iterator_category;
@@ -297,15 +267,18 @@ namespace gmm {
       : it(iter), si1(s1), si2(s2), ii(i) { }
   };
 
-  template <class PT, class SUBI1, class SUBI2>
+  template <typename PT, typename SUBI1, typename SUBI2>
   struct linalg_traits<gen_sub_col_matrix<PT, SUBI1, SUBI2> > {
     typedef gen_sub_col_matrix<PT, SUBI1, SUBI2> this_type;
     typedef typename std::iterator_traits<PT>::value_type M;
+    typedef typename linalg_traits<M>::origin_type origin_type;
+    typedef typename select_ref<const origin_type *, origin_type *,
+			        PT>::ref_type porigin_type;
     typedef typename which_reference<PT>::is_reference is_reference;
     typedef abstract_matrix linalg_type;
     typedef typename linalg_traits<M>::value_type value_type;
-    typedef typename select_return<value_type,
-            typename linalg_traits<M>::reference, PT>::return_type reference;
+    typedef typename select_ref<value_type,
+            typename linalg_traits<M>::reference, PT>::ref_type reference;
     typedef abstract_null_type sub_row_type;
     typedef abstract_null_type row_iterator;
     typedef abstract_null_type const_sub_row_type;
@@ -313,17 +286,17 @@ namespace gmm {
     typedef typename sub_vector_type<const typename
             linalg_traits<M>::const_sub_col_type *, SUBI1>::vector_type
             const_sub_col_type;
-    typedef typename select_return<abstract_null_type, 
+    typedef typename select_ref<abstract_null_type, 
             typename sub_vector_type<typename linalg_traits<M>::sub_col_type *,
-	    SUBI1>::vector_type, PT>::return_type sub_col_type;
+	    SUBI1>::vector_type, PT>::ref_type sub_col_type;
     typedef gen_sub_col_matrix_iterator<typename const_pointer<PT>::pointer,
 	    SUBI1, SUBI2> const_col_iterator;
-    typedef typename select_return<abstract_null_type, 
-	    gen_sub_col_matrix_iterator<PT, SUBI1, SUBI2>, PT>::return_type
+    typedef typename select_ref<abstract_null_type, 
+	    gen_sub_col_matrix_iterator<PT, SUBI1, SUBI2>, PT>::ref_type
             col_iterator;
-    typedef gen_sub_col_matrix_access<PT, SUBI1, SUBI2> access_type;
     typedef col_major sub_orientation;
-    typedef typename linalg_traits<const_sub_col_type>::storage_type storage_type;
+    typedef typename linalg_traits<const_sub_col_type>::storage_type
+    storage_type;
     static size_type nrows(const this_type &m) { return m.nrows(); }
     static size_type ncols(const this_type &m) { return m.ncols(); }
     static const_sub_col_type col(const const_col_iterator &it)
@@ -331,89 +304,103 @@ namespace gmm {
     static sub_col_type col(const col_iterator &it)
     { return sub_col_type(linalg_traits<M>::col(*it), it.si1); }
     static const_col_iterator col_begin(const this_type &m)
-    { return const_col_iterator(m._begin, m.si1, m.si2, 0); }
+    { return const_col_iterator(m.begin_, m.si1, m.si2, 0); }
     static col_iterator col_begin(this_type &m)
-    { return col_iterator(m._begin, m.si1, m.si2, 0); }
+    { return col_iterator(m.begin_, m.si1, m.si2, 0); }
     static const_col_iterator col_end(const this_type &m)
-    { return const_col_iterator(m._begin, m.si1, m.si2,  m.ncols()); }
+    { return const_col_iterator(m.begin_, m.si1, m.si2,  m.ncols()); }
     static col_iterator col_end(this_type &m)
-    { return col_iterator(m._begin, m.si1, m.si2, m.ncols()); }    
-    static const void* origin(const this_type &m) { return m.origin; }
+    { return col_iterator(m.begin_, m.si1, m.si2, m.ncols()); } 
+    static origin_type* origin(this_type &v) { return v.origin; }
+    static const origin_type* origin(const this_type &v) { return v.origin; }
     static void do_clear(this_type &m) {
       col_iterator it = mat_col_begin(m), ite = mat_col_end(m);
       for (; it != ite; ++it) clear(col(it));
     }
+    static value_type access(const const_col_iterator &itcol, size_type i)
+    { return linalg_traits<M>::access(*itcol, itcol.si1.index(i)); }
+    static reference access(const col_iterator &itcol, size_type i)
+    { return linalg_traits<M>::access(*itcol, itcol.si1.index(i)); }
   };
 
-  template <class PT, class SUBI1, class SUBI2> std::ostream &operator <<
+  template <typename PT, typename SUBI1, typename SUBI2> std::ostream &operator <<
   (std::ostream &o, const gen_sub_col_matrix<PT, SUBI1, SUBI2>& m)
   { gmm::write(o,m); return o; }
 
-#ifdef USING_BROKEN_GCC295
-  template <class PT, class SUBI1, class SUBI2>
-  struct linalg_traits<const gen_sub_col_matrix<PT, SUBI1, SUBI2> >
-    : public linalg_traits<gen_sub_col_matrix<PT, SUBI1, SUBI2> > {};
-#endif
-
-
   /* ******************************************************************** */
   /*		sub matrices                                              */
   /* ******************************************************************** */
-
   
-  template <class PT, class SUBI1, class SUBI2, class ST>
-  struct _sub_matrix_type {
+  template <typename PT, typename SUBI1, typename SUBI2, typename ST>
+  struct sub_matrix_type_ {
     typedef abstract_null_type return_type;
   };
-  template <class PT, class SUBI1, class SUBI2>
-  struct _sub_matrix_type<PT, SUBI1, SUBI2, col_major>
+  template <typename PT, typename SUBI1, typename SUBI2>
+  struct sub_matrix_type_<PT, SUBI1, SUBI2, col_major>
   { typedef gen_sub_col_matrix<PT, SUBI1, SUBI2> matrix_type; };
-  template <class PT, class SUBI1, class SUBI2>
-  struct _sub_matrix_type<PT, SUBI1, SUBI2, row_major>
+  template <typename PT, typename SUBI1, typename SUBI2>
+  struct sub_matrix_type_<PT, SUBI1, SUBI2, row_major>
   { typedef gen_sub_row_matrix<PT, SUBI1, SUBI2> matrix_type; };
-  template <class PT, class SUBI1, class SUBI2>
+  template <typename PT, typename SUBI1, typename SUBI2>
   struct sub_matrix_type {
     typedef typename std::iterator_traits<PT>::value_type M;
-    typedef typename _sub_matrix_type<PT, SUBI1, SUBI2,
+    typedef typename sub_matrix_type_<PT, SUBI1, SUBI2,
         typename principal_orientation_type<typename
         linalg_traits<M>::sub_orientation>::potype>::matrix_type matrix_type;
   };
 
-  template <class M, class SUBI1, class SUBI2>  inline
+  template <typename M, typename SUBI1, typename SUBI2>  inline
     typename select_return<typename sub_matrix_type<const M *, SUBI1, SUBI2>
     ::matrix_type, typename sub_matrix_type<M *, SUBI1, SUBI2>::matrix_type,
     M *>::return_type
   sub_matrix(M &m, const SUBI1 &si1, const SUBI2 &si2) {
+#   ifdef GMM_VERIFY
+    if (si1.last() > mat_nrows(m) || si2.last() > mat_ncols(m))
+      DAL_THROW(dimension_error, "sub matrix too large");
+#   endif
     return typename select_return<typename sub_matrix_type<const M *, SUBI1,
       SUBI2>::matrix_type, typename sub_matrix_type<M *, SUBI1, SUBI2>
       ::matrix_type, M *>::return_type(linalg_cast(m), si1, si2);
   }
 
-  template <class M, class SUBI1, class SUBI2>  inline
+  template <typename M, typename SUBI1, typename SUBI2>  inline
     typename select_return<typename sub_matrix_type<const M *, SUBI1, SUBI2>
     ::matrix_type, typename sub_matrix_type<M *, SUBI1, SUBI2>::matrix_type,
     const M *>::return_type
   sub_matrix(const M &m, const SUBI1 &si1, const SUBI2 &si2) {
+#   ifdef GMM_VERIFY
+    if (si1.last() > mat_nrows(m) || si2.last() > mat_ncols(m))
+      DAL_THROW(dimension_error, "sub matrix too large");
+#   endif
+
     return typename select_return<typename sub_matrix_type<const M *, SUBI1,
       SUBI2>::matrix_type, typename sub_matrix_type<M *, SUBI1, SUBI2>
       ::matrix_type, const M *>::return_type(linalg_cast(m), si1, si2);
   }
 
-  template <class M, class SUBI1>  inline
+  template <typename M, typename SUBI1>  inline
     typename select_return<typename sub_matrix_type<const M *, SUBI1, SUBI1>
     ::matrix_type, typename sub_matrix_type<M *, SUBI1, SUBI1>::matrix_type,
     M *>::return_type
   sub_matrix(M &m, const SUBI1 &si1) {
+#   ifdef GMM_VERIFY
+    if (si1.last() > mat_nrows(m) || si1.last() > mat_ncols(m))
+      DAL_THROW(dimension_error, "sub matrix too large");
+#   endif
     return typename select_return<typename sub_matrix_type<const M *, SUBI1,
       SUBI1>::matrix_type, typename sub_matrix_type<M *, SUBI1, SUBI1>
       ::matrix_type, M *>::return_type(linalg_cast(m), si1, si1);
   }
 
-  template <class M, class SUBI1>  inline
+  template <typename M, typename SUBI1>  inline
     typename select_return<typename sub_matrix_type<const M *, SUBI1, SUBI1>
     ::matrix_type, typename sub_matrix_type<M *, SUBI1, SUBI1>::matrix_type,
     const M *>::return_type
   sub_matrix(const M &m, const SUBI1 &si1) {
+#   ifdef GMM_VERIFY
+    if (si1.last() > mat_nrows(m) || si1.last() > mat_ncols(m))
+      DAL_THROW(dimension_error, "sub matrix too large");
+#   endif
     return typename select_return<typename sub_matrix_type<const M *, SUBI1,
       SUBI1>::matrix_type, typename sub_matrix_type<M *, SUBI1, SUBI1>
       ::matrix_type, const M *>::return_type(linalg_cast(m), si1, si1);
@@ -422,4 +409,4 @@ namespace gmm {
 
 }
 
-#endif //  __GMM_SUB_MATRIX_H
+#endif //  GMM_SUB_MATRIX_H__
