@@ -9,6 +9,6 @@ while (<F>) {
   }
 
 }
+close(F); if ($?) { exit(1); }
 if ($er == 1) { exit(1); }
-`./poly`;
-if ($?) { exit(1); }
+
