@@ -31,7 +31,6 @@ while (<F>) {
   }
 
 }
+close(F); if ($?) { `rm -f $tmp`; exit(1); }
 if ($er == 1) { `rm -f $tmp`; exit(1); }
-`./test_link_fem $tmp`;
 `rm -f $tmp`;
-if ($?) { exit(1); }
